import datetime as dt
from _typeshed import ConvertibleToInt, Incomplete, SupportsRead, SupportsReadline, SupportsWrite, Unused
from collections.abc import Callable, Iterable, Mapping, Sequence
from types import TracebackType
from typing import Any, Literal, NoReturn, Protocol, TextIO, TypeAlias, TypeVar, overload, type_check_only
from typing_extensions import Self, disjoint_base

from psycopg2.extras import ReplicationCursor as extras_ReplicationCursor
from psycopg2.sql import Composable

_Vars: TypeAlias = Sequence[Any] | Mapping[str, Any] | None

@type_check_only
class _type:
    # The class doesn't exist at runtime but following attributes have type "psycopg2._psycopg.type"
    name: str
    values: tuple[int, ...]
    def __call__(self, value: str | bytes | None, cur: cursor | None, /) -> Any: ...

BINARY: _type
BINARYARRAY: _type
BOOLEAN: _type
BOOLEANARRAY: _type
BYTES: _type
BYTESARRAY: _type
CIDRARRAY: _type
DATE: _type
DATEARRAY: _type
DATETIME: _type
DATETIMEARRAY: _type
DATETIMETZ: _type
DATETIMETZARRAY: _type
DECIMAL: _type
DECIMALARRAY: _type
FLOAT: _type
FLOATARRAY: _type
INETARRAY: _type
INTEGER: _type
INTEGERARRAY: _type
INTERVAL: _type
INTERVALARRAY: _type
LONGINTEGER: _type
LONGINTEGERARRAY: _type
MACADDRARRAY: _type
NUMBER: _type
PYDATE: _type
PYDATEARRAY: _type
PYDATETIME: _type
PYDATETIMEARRAY: _type
PYDATETIMETZ: _type
PYDATETIMETZARRAY: _type
PYINTERVAL: _type
PYINTERVALARRAY: _type
PYTIME: _type
PYTIMEARRAY: _type
ROWID: _type
ROWIDARRAY: _type
STRING: _type
STRINGARRAY: _type
TIME: _type
TIMEARRAY: _type
UNICODE: _type
UNICODEARRAY: _type
UNKNOWN: _type

REPLICATION_LOGICAL: int
REPLICATION_PHYSICAL: int

@type_check_only
class _ISQLQuoteProto(Protocol):
    # Objects conforming this protocol should implement a getquoted() and optionally a prepare() method.
    # The real ISQLQuote class is implemented below with more stuff.
    def getquoted(self) -> bytes: ...
    # def prepare(self, __conn: connection) -> None: ...  # optional

adapters: dict[tuple[type[Any], type[ISQLQuote]], Callable[[Any], _ISQLQuoteProto]]
apilevel: str
binary_types: dict[Any, Any]
encodings: dict[str, str]
paramstyle: str
sqlstate_errors: dict[str, type[Error]]
string_types: dict[int, _type]
threadsafety: int

__libpq_version__: int

_T_co = TypeVar("_T_co", covariant=True)

@type_check_only
class _SupportsReadAndReadline(SupportsRead[_T_co], SupportsReadline[_T_co], Protocol[_T_co]): ...

@disjoint_base
class cursor:
    arraysize: int
    binary_types: Incomplete | None
    connection: _Connection
    itersize: int
    row_factory: Incomplete | None
    scrollable: bool | None
    string_types: Incomplete | None
    tzinfo_factory: Callable[..., dt.tzinfo]
    withhold: bool
    def __init__(self, conn: _Connection, name: str | bytes | None = None) -> None: ...
    @property
    def closed(self) -> bool: ...
    @property
    def lastrowid(self) -> int: ...
    @property
    def name(self) -> Incomplete | None: ...
    @property
    def query(self) -> bytes | None: ...
    @property
    def description(self) -> tuple[Column, ...] | None: ...
    @property
    def rowcount(self) -> int: ...
    @property
    def rownumber(self) -> int: ...
    @property
    def typecaster(self) -> Incomplete | None: ...
    @property
    def statusmessage(self) -> str | None: ...
    @property
    def pgresult_ptr(self) -> int | None: ...
    def callproc(self, procname: str | bytes, parameters: _Vars = None, /) -> None: ...
    def cast(self, oid: int, s: str | bytes, /) -> Any: ...
    def close(self) -> None: ...
    def copy_expert(
        self,
        sql: str | bytes | Composable,
        file: _SupportsReadAndReadline[bytes] | SupportsWrite[bytes] | TextIO,
        size: int = 8192,
    ) -> None: ...
    def copy_from(
        self,
        file: _SupportsReadAndReadline[bytes] | _SupportsReadAndReadline[str],
        table: str,
        sep: str = "\t",
        null: str = "\\N",
        size: int = 8192,
        columns: Iterable[str] | None = None,
    ) -> None: ...
    def copy_to(
        self,
        file: SupportsWrite[bytes] | TextIO,
        table: str,
        sep: str = "\t",
        null: str = "\\N",
        columns: Iterable[str] | None = None,
    ) -> None: ...
    def execute(self, query: str | bytes | Composable, vars: _Vars = None) -> None: ...
    def executemany(self, query: str | bytes | Composable, vars_list: Iterable[_Vars]) -> None: ...
    def fetchall(self) -> list[tuple[Any, ...]]: ...
    def fetchmany(self, size: int | None = None) -> list[tuple[Any, ...]]: ...
    def fetchone(self) -> tuple[Any, ...] | None: ...
    def mogrify(self, query: str | bytes | Composable, vars: _Vars | None = None) -> bytes: ...
    def nextset(self) -> NoReturn: ...  # not supported
    def scroll(self, value: int, mode: Literal["absolute", "relative"] = "relative") -> None: ...
    def setinputsizes(self, sizes: Unused) -> None: ...
    def setoutputsize(self, size: int, column: int = ..., /) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, type: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> tuple[Any, ...]: ...

_Cursor: TypeAlias = cursor

@disjoint_base
class AsIs:
    def __init__(self, obj: object, /, **kwargs: Unused) -> None: ...
    @property
    def adapted(self) -> Any: ...
    def getquoted(self) -> bytes: ...
    def __conform__(self, proto, /) -> Self | None: ...

@disjoint_base
class Binary:
    def __init__(self, str: object, /, **kwargs: Unused) -> None: ...
    @property
    def adapted(self) -> Any: ...
    @property
    def buffer(self) -> Any: ...
    def getquoted(self) -> bytes: ...
    def prepare(self, conn: connection, /) -> None: ...
    def __conform__(self, proto, /) -> Self | None: ...

@disjoint_base
class Boolean:
    def __init__(self, obj: object, /, **kwargs: Unused) -> None: ...
    @property
    def adapted(self) -> Any: ...
    def getquoted(self) -> bytes: ...
    def __conform__(self, proto, /) -> Self | None: ...

@disjoint_base
class Column:
    display_size: Any
    internal_size: Any
    name: Any
    null_ok: Any
    precision: Any
    scale: Any
    table_column: Any
    table_oid: Any
    type_code: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def __eq__(self, other, /): ...
    def __ge__(self, other, /): ...
    def __getitem__(self, index, /): ...
    def __getstate__(self): ...
    def __gt__(self, other, /): ...
    def __le__(self, other, /): ...
    def __len__(self) -> int: ...
    def __lt__(self, other, /): ...
    def __ne__(self, other, /): ...
    def __setstate__(self, state, /): ...

@disjoint_base
class ConnectionInfo:
    # Note: the following properties can be None if their corresponding libpq function
    # returns NULL. They're not annotated as such, because this is very unlikely in
    # practice---the psycopg2 docs [1] don't even mention this as a possibility!
    #
    # - db_name
    # - user
    # - password
    # - host
    # - port
    # - options
    #
    # (To prove this, one needs to inspect the psycopg2 source code [2], plus the
    # documentation [3] and source code [4] of the corresponding libpq calls.)
    #
    # [1]: https://www.psycopg.org/docs/extensions.html#psycopg2.extensions.ConnectionInfo
    # [2]: https://github.com/psycopg/psycopg2/blob/1d3a89a0bba621dc1cc9b32db6d241bd2da85ad1/psycopg/conninfo_type.c#L52 and below
    # [3]: https://www.postgresql.org/docs/current/libpq-status.html
    # [4]: https://github.com/postgres/postgres/blob/b39838889e76274b107935fa8e8951baf0e8b31b/src/interfaces/libpq/fe-connect.c#L6754 and below  # noqa: E501
    @property
    def backend_pid(self) -> int: ...
    @property
    def dbname(self) -> str: ...
    @property
    def dsn_parameters(self) -> dict[str, str]: ...
    @property
    def error_message(self) -> str | None: ...
    @property
    def host(self) -> str: ...
    @property
    def needs_password(self) -> bool: ...
    @property
    def options(self) -> str: ...
    @property
    def password(self) -> str: ...
    @property
    def port(self) -> int: ...
    @property
    def protocol_version(self) -> int: ...
    @property
    def server_version(self) -> int: ...
    @property
    def socket(self) -> int: ...
    @property
    def ssl_attribute_names(self) -> list[str]: ...
    @property
    def ssl_in_use(self) -> bool: ...
    @property
    def status(self) -> int: ...
    @property
    def transaction_status(self) -> int: ...
    @property
    def used_password(self) -> bool: ...
    @property
    def user(self) -> str: ...
    def __init__(self, *args, **kwargs) -> None: ...
    def parameter_status(self, name: str) -> str | None: ...
    def ssl_attribute(self, name: str) -> str | None: ...

@disjoint_base
class Error(Exception):
    cursor: _Cursor | None
    diag: Diagnostics
    pgcode: str | None
    pgerror: str | None
    def __init__(self, *args, **kwargs) -> None: ...
    def __reduce__(self): ...
    def __setstate__(self, state, /): ...

class DatabaseError(Error): ...
class DataError(DatabaseError): ...
class IntegrityError(DatabaseError): ...
class InternalError(DatabaseError): ...
class NotSupportedError(DatabaseError): ...
class OperationalError(DatabaseError): ...
class ProgrammingError(DatabaseError): ...
class QueryCanceledError(OperationalError): ...
class TransactionRollbackError(OperationalError): ...
class InterfaceError(Error): ...
class Warning(Exception): ...

@disjoint_base
class ISQLQuote:
    _wrapped: Any
    def __init__(self, wrapped: object, /, **kwargs) -> None: ...
    def getbinary(self): ...
    def getbuffer(self): ...
    def getquoted(self) -> bytes: ...

@disjoint_base
class Decimal:
    def __init__(self, value: object, /, **kwargs: Unused) -> None: ...
    @property
    def adapted(self) -> Any: ...
    def getquoted(self) -> bytes: ...
    def __conform__(self, proto, /) -> Self | None: ...

@disjoint_base
class Diagnostics:
    column_name: str | None
    constraint_name: str | None
    context: str | None
    datatype_name: str | None
    internal_position: str | None
    internal_query: str | None
    message_detail: str | None
    message_hint: str | None
    message_primary: str | None
    schema_name: str | None
    severity: str | None
    severity_nonlocalized: str | None
    source_file: str | None
    source_function: str | None
    source_line: str | None
    sqlstate: str | None
    statement_position: str | None
    table_name: str | None
    def __init__(self, err: Error, /) -> None: ...

@disjoint_base
class Float:
    def __init__(self, value: float, /, **kwargs: Unused) -> None: ...
    @property
    def adapted(self) -> float: ...
    def getquoted(self) -> bytes: ...
    def __conform__(self, proto, /) -> Self | None: ...

@disjoint_base
class Int:
    def __init__(self, value: ConvertibleToInt, /, **kwargs: Unused) -> None: ...
    @property
    def adapted(self) -> Any: ...
    def getquoted(self) -> bytes: ...
    def __conform__(self, proto, /) -> Self | None: ...

@disjoint_base
class List:
    def __init__(self, objs: list[object], /, **kwargs: Unused) -> None: ...
    @property
    def adapted(self) -> list[Any]: ...
    def getquoted(self) -> bytes: ...
    def prepare(self, conn: connection, /) -> None: ...
    def __conform__(self, proto, /) -> Self | None: ...

@disjoint_base
class Notify:
    channel: Any
    payload: Any
    pid: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def __eq__(self, other, /): ...
    def __ge__(self, other, /): ...
    def __getitem__(self, index, /): ...
    def __gt__(self, other, /): ...
    def __hash__(self) -> int: ...
    def __le__(self, other, /): ...
    def __len__(self) -> int: ...
    def __lt__(self, other, /): ...
    def __ne__(self, other, /): ...

@disjoint_base
class QuotedString:
    encoding: str
    def __init__(self, str: object, /, **kwargs: Unused) -> None: ...
    @property
    def adapted(self) -> Any: ...
    @property
    def buffer(self) -> Any: ...
    def getquoted(self) -> bytes: ...
    def prepare(self, conn: connection, /) -> None: ...
    def __conform__(self, proto, /) -> Self | None: ...

@disjoint_base
class ReplicationCursor(cursor):
    feedback_timestamp: Any
    io_timestamp: Any
    wal_end: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def consume_stream(self, consumer, keepalive_interval=...): ...
    def read_message(self) -> Incomplete | None: ...
    def send_feedback(self, write_lsn=..., flush_lsn=..., apply_lsn=..., reply=..., force=...): ...
    def start_replication_expert(self, command, decode=..., status_interval=...): ...

@disjoint_base
class ReplicationMessage:
    cursor: Any
    data_size: Any
    data_start: Any
    payload: Any
    send_time: Any
    wal_end: Any
    def __init__(self, *args, **kwargs) -> None: ...

@disjoint_base
class Xid:
    bqual: Any
    database: Any
    format_id: Any
    gtrid: Any
    owner: Any
    prepared: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def from_string(self, *args, **kwargs): ...
    def __getitem__(self, index, /): ...
    def __len__(self) -> int: ...

_T_cur = TypeVar("_T_cur", bound=cursor)

@disjoint_base
class connection:
    DataError: type[DataError]
    DatabaseError: type[DatabaseError]
    Error: type[Error]
    IntegrityError: type[IntegrityError]
    InterfaceError: type[InterfaceError]
    InternalError: type[InternalError]
    NotSupportedError: type[NotSupportedError]
    OperationalError: type[OperationalError]
    ProgrammingError: type[ProgrammingError]
    Warning: type[Warning]
    @property
    def async_(self) -> int: ...
    autocommit: bool
    @property
    def binary_types(self) -> dict[Incomplete, Incomplete]: ...
    @property
    def closed(self) -> int: ...
    cursor_factory: Callable[[connection, str | bytes | None], cursor]
    @property
    def dsn(self) -> str: ...
    @property
    def encoding(self) -> str: ...
    @property
    def info(self) -> ConnectionInfo: ...

    @property
    def isolation_level(self) -> int | None: ...
    @isolation_level.setter
    def isolation_level(self, value: str | bytes | int | None, /) -> None: ...

    notices: list[str]
    notifies: list[Notify]
    @property
    def pgconn_ptr(self) -> int | None: ...
    @property
    def protocol_version(self) -> int: ...

    @property
    def deferrable(self) -> bool | None: ...
    @deferrable.setter
    def deferrable(self, value: Literal["default"] | bool | None, /) -> None: ...

    @property
    def readonly(self) -> bool | None: ...
    @readonly.setter
    def readonly(self, value: Literal["default"] | bool | None, /) -> None: ...

    @property
    def server_version(self) -> int: ...
    @property
    def status(self) -> int: ...
    @property
    def string_types(self) -> dict[Incomplete, Incomplete]: ...
    # Really it's dsn: str, async: int = 0, async_: int = 0, but
    # that would be a syntax error.
    def __init__(self, dsn: str, *, async_: int = 0) -> None: ...
    def cancel(self) -> None: ...
    def close(self) -> None: ...
    def commit(self) -> None: ...

    @overload
    def cursor(
        self, name: str | bytes | None = None, cursor_factory: None = None, withhold: bool = False, scrollable: bool | None = None
    ) -> cursor: ...
    @overload
    def cursor(
        self,
        name: str | bytes | None = None,
        *,
        cursor_factory: Callable[[connection, str | bytes | None], _T_cur],
        withhold: bool = False,
        scrollable: bool | None = None,
    ) -> _T_cur: ...
    @overload
    def cursor(
        self,
        name: str | bytes | None,
        cursor_factory: Callable[[connection, str | bytes | None], _T_cur],
        withhold: bool = False,
        scrollable: bool | None = None,
    ) -> _T_cur: ...

    def fileno(self) -> int: ...
    def get_backend_pid(self) -> int: ...
    def get_dsn_parameters(self) -> dict[str, str]: ...
    def get_native_connection(self): ...
    def get_parameter_status(self, parameter: str) -> str | None: ...
    def get_transaction_status(self) -> int: ...
    def isexecuting(self) -> bool: ...
    def lobject(
        self,
        oid: int = ...,
        mode: str | None = ...,
        new_oid: int = ...,
        new_file: str | None = ...,
        lobject_factory: type[lobject] = ...,
    ) -> lobject: ...
    def poll(self) -> int: ...
    def reset(self) -> None: ...
    def rollback(self) -> None: ...
    def set_client_encoding(self, encoding: str) -> None: ...
    def set_isolation_level(self, level: int | None) -> None: ...
    def set_session(
        self,
        isolation_level: str | bytes | int | None = ...,
        readonly: bool | Literal["default", b"default"] | None = ...,
        deferrable: bool | Literal["default", b"default"] | None = ...,
        autocommit: bool = ...,
    ) -> None: ...
    def tpc_begin(self, xid: str | bytes | Xid) -> None: ...
    def tpc_commit(self, xid: str | bytes | Xid = ..., /) -> None: ...
    def tpc_prepare(self) -> None: ...
    def tpc_recover(self) -> list[Xid]: ...
    def tpc_rollback(self, xid: str | bytes | Xid = ..., /) -> None: ...
    def xid(self, format_id, gtrid, bqual) -> Xid: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, type: type[BaseException] | None, name: BaseException | None, tb: TracebackType | None, /) -> None: ...

_Connection: TypeAlias = connection

@disjoint_base
class ReplicationConnection(connection):
    autocommit: Any
    isolation_level: Any
    replication_type: Any
    reset: Any
    set_isolation_level: Any
    set_session: Any
    def __init__(self, *args, **kwargs) -> None: ...

    # https://github.com/python/typeshed/issues/11282
    # The return type should be exactly extras.ReplicationCursor (not _psycopg.ReplicationCursor)
    # See the C code: replicationConnection_init(), psyco_conn_cursor()
    @overload
    def cursor(
        self, name: str | bytes | None = None, cursor_factory: None = None, withhold: bool = False, scrollable: bool | None = None
    ) -> extras_ReplicationCursor: ...
    @overload
    def cursor(
        self,
        name: str | bytes | None = None,
        *,
        cursor_factory: Callable[[connection, str | bytes | None], _T_cur],
        withhold: bool = False,
        scrollable: bool | None = None,
    ) -> _T_cur: ...
    @overload
    def cursor(
        self,
        name: str | bytes | None,
        cursor_factory: Callable[[connection, str | bytes | None], _T_cur],
        withhold: bool = False,
        scrollable: bool | None = None,
    ) -> _T_cur: ...

@disjoint_base
class lobject:
    closed: Any
    mode: Any
    oid: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def close(self): ...
    def export(self, filename): ...
    def read(self, size=...): ...
    def seek(self, offset, whence=...): ...
    def tell(self): ...
    def truncate(self, len=...): ...
    def unlink(self): ...
    def write(self, str): ...

@type_check_only
class _datetime:
    # The class doesn't exist at runtime but functions below return "psycopg2._psycopg.datetime" objects
    # XXX: This and other classes that implement the `ISQLQuote` protocol could be made generic
    # in the return type of their `adapted` property if someone asks for it.
    def __init__(self, obj: object, type: int = -1, /, **kwargs: Unused) -> None: ...
    @property
    def adapted(self) -> Any: ...
    @property
    def type(self) -> int: ...
    def getquoted(self) -> bytes: ...
    def __conform__(self, proto, /) -> Self | None: ...

def Date(year: int, month: int, day: int, /) -> _datetime: ...
def DateFromPy(date: dt.date, /) -> _datetime: ...
def DateFromTicks(ticks: float, /) -> _datetime: ...
def IntervalFromPy(interval: dt.timedelta, /) -> _datetime: ...
def Time(hour: int, minutes: int, seconds: float, tzinfo: dt.tzinfo | None = None, /) -> _datetime: ...
def TimeFromPy(time: dt.time, /) -> _datetime: ...
def TimeFromTicks(ticks: float, /) -> _datetime: ...
def Timestamp(
    year: int, month: int, day: int, hour: int = 0, minutes: int = 0, seconds: float = 0, tzinfo: dt.tzinfo | None = None, /
) -> _datetime: ...
def TimestampFromPy(datetime: dt.datetime, /) -> _datetime: ...
def TimestampFromTicks(ticks: float, /) -> _datetime: ...
def _connect(*args, **kwargs): ...
def adapt(obj: object, protocol=..., alternate=..., /) -> Any: ...
def encrypt_password(
    password: str | bytes, user: str | bytes, scope: connection | cursor | None = None, algorithm: str | None = None
) -> str: ...
def get_wait_callback() -> Incomplete | None: ...
def libpq_version() -> int: ...
def new_array_type(values: tuple[int, ...], name: str, baseobj: _type) -> _type: ...
def new_type(
    values: tuple[int, ...], name: str, castobj: Callable[[str | bytes | None, cursor], Any] | None = None, baseobj=None
) -> _type: ...
def parse_dsn(dsn: str | bytes) -> dict[str, Any]: ...
def quote_ident(ident: str | bytes, scope) -> str: ...
def register_type(obj: _type, conn_or_curs: connection | cursor | None = None, /) -> None: ...
def set_wait_callback(none: Callable[..., Incomplete] | None, /) -> None: ...



from _typeshed import Incomplete, SupportsWrite
from collections.abc import Iterable
from typing import TypeVar

from pygments.formatter import Formatter
from pygments.token import _TokenType

_T = TypeVar("_T", str, bytes)

__all__ = ["Terminal256Formatter", "TerminalTrueColorFormatter"]

class EscapeSequence:
    fg: Incomplete
    bg: Incomplete
    bold: Incomplete
    underline: Incomplete
    italic: Incomplete
    def __init__(self, fg=None, bg=None, bold: bool = False, underline: bool = False, italic: bool = False) -> None: ...
    def escape(self, attrs): ...
    def color_string(self): ...
    def true_color_string(self): ...
    def reset_string(self): ...

class Terminal256Formatter(Formatter[_T]):
    xterm_colors: Incomplete
    best_match: Incomplete
    style_string: Incomplete
    usebold: Incomplete
    useunderline: Incomplete
    useitalic: Incomplete
    linenos: Incomplete

    def format(self, tokensource: Iterable[tuple[_TokenType, str]], outfile: SupportsWrite[_T]) -> None: ...
    def format_unencoded(self, tokensource: Iterable[tuple[_TokenType, str]], outfile: SupportsWrite[str]) -> None: ...

class TerminalTrueColorFormatter(Terminal256Formatter[_T]): ...



from _typeshed import Incomplete

from kafka.protocol.api import Request, Response

UNKNOWN_OFFSET: int

class OffsetResetStrategy:
    LATEST: int
    EARLIEST: int
    NONE: int

class ListOffsetsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ListOffsetsResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ListOffsetsResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ListOffsetsResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ListOffsetsResponse_v4(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ListOffsetsResponse_v5(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ListOffsetsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ListOffsetsResponse_v0
    SCHEMA: Incomplete
    DEFAULTS: Incomplete

class ListOffsetsRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ListOffsetsResponse_v1
    SCHEMA: Incomplete
    DEFAULTS: Incomplete

class ListOffsetsRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ListOffsetsResponse_v2
    SCHEMA: Incomplete
    DEFAULTS: Incomplete

class ListOffsetsRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ListOffsetsResponse_v3
    SCHEMA: Incomplete
    DEFAULTS: Incomplete

class ListOffsetsRequest_v4(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ListOffsetsResponse_v4
    SCHEMA: Incomplete
    DEFAULTS: Incomplete

class ListOffsetsRequest_v5(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ListOffsetsResponse_v5
    SCHEMA: Incomplete
    DEFAULTS: Incomplete

ListOffsetsRequest: Incomplete
ListOffsetsResponse: Incomplete



import sys
from collections.abc import Callable
from typing import Any

__all__ = ["JSONDecoder", "JSONDecodeError"]

class JSONDecodeError(ValueError):
    msg: str
    doc: str
    pos: int
    lineno: int
    colno: int
    def __init__(self, msg: str, doc: str, pos: int) -> None: ...

class JSONDecoder:
    if sys.version_info >= (3, 15):
        array_hook: Callable[[list[Any]], Any] | None
    object_hook: Callable[[dict[str, Any]], Any]
    parse_float: Callable[[str], Any]
    parse_int: Callable[[str], Any]
    parse_constant: Callable[[str], Any]
    strict: bool
    object_pairs_hook: Callable[[list[tuple[str, Any]]], Any]
    if sys.version_info >= (3, 15):
        def __init__(
            self,
            *,
            object_hook: Callable[[dict[str, Any]], Any] | None = None,
            parse_float: Callable[[str], Any] | None = None,
            parse_int: Callable[[str], Any] | None = None,
            parse_constant: Callable[[str], Any] | None = None,
            strict: bool = True,
            object_pairs_hook: Callable[[list[tuple[str, Any]]], Any] | None = None,
            array_hook: Callable[[list[Any]], Any] | None = None,
        ) -> None: ...

    else:
        def __init__(
            self,
            *,
            object_hook: Callable[[dict[str, Any]], Any] | None = None,
            parse_float: Callable[[str], Any] | None = None,
            parse_int: Callable[[str], Any] | None = None,
            parse_constant: Callable[[str], Any] | None = None,
            strict: bool = True,
            object_pairs_hook: Callable[[list[tuple[str, Any]]], Any] | None = None,
        ) -> None: ...

    def decode(self, s: str, _w: Callable[..., Any] = ...) -> Any: ...  # _w is undocumented
    def raw_decode(self, s: str, idx: int = 0) -> tuple[Any, int]: ...



from collections.abc import Mapping

import requests.adapters
from urllib3.connectionpool import ConnectionPool

class BaseHTTPAdapter(requests.adapters.HTTPAdapter):
    def close(self) -> None: ...
    def get_connection_with_tls_context(
        self,
        request: requests.PreparedRequest,
        verify: bool | str | None,
        proxies: Mapping[str, str] | None = None,
        cert: tuple[str, str] | str | None = None,
    ) -> ConnectionPool: ...



import abc
from _typeshed import Incomplete

from kafka.metrics.measurable_stat import AbstractMeasurableStat

class AbstractSampledStat(AbstractMeasurableStat, metaclass=abc.ABCMeta):
    def __init__(self, initial_value) -> None: ...
    @abc.abstractmethod
    def update(self, sample, config, value, time_ms): ...
    @abc.abstractmethod
    def combine(self, samples, config, now): ...
    def record(self, config, value, time_ms) -> None: ...
    def new_sample(self, time_ms): ...
    def measure(self, config, now): ...
    def current(self, time_ms): ...
    def oldest(self, now): ...
    def purge_obsolete_samples(self, config, now) -> None: ...

    class Sample:
        initial_value: Incomplete
        event_count: int
        last_window_ms: Incomplete
        value: Incomplete
        def __init__(self, initial_value, now) -> None: ...
        def reset(self, now) -> None: ...
        def is_complete(self, time_ms, config): ...



from _typeshed import StrOrBytesPath, structseq
from collections.abc import Callable
from typing import Final, TypeAlias, final
from typing_extensions import Self

_Location: TypeAlias = tuple[int, int, int, int] | LocationInfo | None
_Frame: TypeAlias = tuple[str, _Location, str, int | None] | FrameInfo
_Stats: TypeAlias = dict[str, int | float]

PROCESS_VM_READV_SUPPORTED: Final[int]
THREAD_STATUS_GIL_REQUESTED: Final[int]
THREAD_STATUS_HAS_EXCEPTION: Final[int]
THREAD_STATUS_HAS_GIL: Final[int]
THREAD_STATUS_MAIN_THREAD: Final[int]
THREAD_STATUS_ON_CPU: Final[int]
THREAD_STATUS_UNKNOWN: Final[int]

@final
class LocationInfo(structseq[int], tuple[int, int, int, int]):
    __match_args__: Final = ("lineno", "end_lineno", "col_offset", "end_col_offset")
    @property
    def lineno(self) -> int: ...
    @property
    def end_lineno(self) -> int: ...
    @property
    def col_offset(self) -> int: ...
    @property
    def end_col_offset(self) -> int: ...

@final
class FrameInfo(structseq[object], tuple[str, _Location, str, int | None]):
    __match_args__: Final = ("filename", "location", "funcname", "opcode")
    @property
    def filename(self) -> str: ...
    @property
    def location(self) -> _Location: ...
    @property
    def funcname(self) -> str: ...
    @property
    def opcode(self) -> int | None: ...

@final
class CoroInfo(structseq[object], tuple[list[_Frame], int | str]):
    __match_args__: Final = ("call_stack", "task_name")
    @property
    def call_stack(self) -> list[_Frame]: ...
    @property
    def task_name(self) -> int | str: ...

@final
class TaskInfo(structseq[object], tuple[int, str, list[CoroInfo], list[CoroInfo]]):
    __match_args__: Final = ("task_id", "task_name", "coroutine_stack", "awaited_by")
    @property
    def task_id(self) -> int: ...
    @property
    def task_name(self) -> str: ...
    @property
    def coroutine_stack(self) -> list[CoroInfo]: ...
    @property
    def awaited_by(self) -> list[CoroInfo]: ...

@final
class ThreadInfo(structseq[object], tuple[int, int, list[_Frame]]):
    __match_args__: Final = ("thread_id", "status", "frame_info")
    @property
    def thread_id(self) -> int: ...
    @property
    def status(self) -> int: ...
    @property
    def frame_info(self) -> list[_Frame]: ...

@final
class InterpreterInfo(structseq[object], tuple[int, list[ThreadInfo]]):
    __match_args__: Final = ("interpreter_id", "threads")
    @property
    def interpreter_id(self) -> int: ...
    @property
    def threads(self) -> list[ThreadInfo]: ...

@final
class AwaitedInfo(structseq[object], tuple[int, list[TaskInfo]]):
    __match_args__: Final = ("thread_id", "awaited_by")
    @property
    def thread_id(self) -> int: ...
    @property
    def awaited_by(self) -> list[TaskInfo]: ...

@final
class GCStatsInfo(structseq[object], tuple[int, int, int, int, int, int, int, int, int, float]):
    __match_args__: Final = (
        "gen",
        "iid",
        "ts_start",
        "ts_stop",
        "collections",
        "collected",
        "uncollectable",
        "candidates",
        "heap_size",
        "duration",
    )
    @property
    def gen(self) -> int: ...
    @property
    def iid(self) -> int: ...
    @property
    def ts_start(self) -> int: ...
    @property
    def ts_stop(self) -> int: ...
    @property
    def collections(self) -> int: ...
    @property
    def collected(self) -> int: ...
    @property
    def uncollectable(self) -> int: ...
    @property
    def candidates(self) -> int: ...
    @property
    def heap_size(self) -> int: ...
    @property
    def duration(self) -> float: ...

@final
class RemoteUnwinder:
    def __init__(
        self,
        pid: int,
        *,
        all_threads: bool = False,
        only_active_thread: bool = False,
        mode: int = 0,
        debug: bool = False,
        skip_non_matching_threads: bool = True,
        native: bool = False,
        gc: bool = False,
        opcodes: bool = False,
        cache_frames: bool = False,
        stats: bool = False,
    ) -> None: ...
    def get_stack_trace(self) -> list[InterpreterInfo]: ...
    def get_all_awaited_by(self) -> list[AwaitedInfo]: ...
    def get_async_stack_trace(self) -> list[AwaitedInfo]: ...
    def get_stats(self) -> _Stats: ...
    def pause_threads(self) -> bool: ...
    def resume_threads(self) -> bool: ...

@final
class GCMonitor:
    def __init__(self, pid: int, *, debug: bool = False) -> None: ...
    def get_gc_stats(self, all_interpreters: bool = False) -> list[GCStatsInfo]: ...

@final
class BinaryWriter:
    def __init__(
        self, filename: StrOrBytesPath, sample_interval_us: int, start_time_us: int, *, compression: int = 0
    ) -> None: ...
    @property
    def total_samples(self) -> int: ...
    def write_sample(self, stack_frames: list[InterpreterInfo], timestamp_us: int) -> None: ...
    def finalize(self) -> None: ...
    def close(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, exc_type: object = None, exc_val: object = None, exc_tb: object = None) -> bool: ...
    def get_stats(self) -> _Stats: ...

@final
class BinaryReader:
    def __init__(self, filename: StrOrBytesPath) -> None: ...
    @property
    def sample_count(self) -> int: ...
    @property
    def sample_interval_us(self) -> int: ...
    def replay(self, collector: object, progress_callback: Callable[[int, int], object] | None = None) -> int: ...
    def get_info(self) -> dict[str, object]: ...
    def get_stats(self) -> _Stats: ...
    def close(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, exc_type: object = None, exc_val: object = None, exc_tb: object = None) -> bool: ...

def zstd_available() -> bool: ...
def get_child_pids(pid: int, *, recursive: bool = True) -> list[int]: ...
def is_python_process(pid: int) -> bool: ...
def get_gc_stats(pid: int, *, all_interpreters: bool = False) -> list[GCStatsInfo]: ...



from _typeshed import Incomplete

log: Incomplete

class Heartbeat:
    DEFAULT_CONFIG: Incomplete
    config: Incomplete
    last_send: Incomplete
    last_receive: Incomplete
    last_poll: Incomplete
    last_reset: Incomplete
    heartbeat_failed: Incomplete
    def __init__(self, **configs) -> None: ...
    def poll(self) -> None: ...
    def sent_heartbeat(self) -> None: ...
    def fail_heartbeat(self) -> None: ...
    def received_heartbeat(self) -> None: ...
    def time_to_next_heartbeat(self): ...
    def should_heartbeat(self): ...
    def session_timeout_expired(self): ...
    def reset_timeouts(self) -> None: ...
    def poll_timeout_expired(self): ...



from decimal import Decimal
from typing import TypedDict, type_check_only

@type_check_only
class _GraphqlVariables(TypedDict, total=False):
    value: str
    currencyCode: str

class MonetaryAmountInput:
    def __init__(self, value: Decimal | None = None, currency_code: str | None = None) -> None: ...
    def to_graphql_variables(self) -> _GraphqlVariables: ...



from _typeshed import Incomplete, SupportsWrite
from collections.abc import Iterable
from typing import TypeVar

from pygments.formatter import Formatter
from pygments.token import _TokenType

_T = TypeVar("_T", str, bytes)

__all__ = ["NullFormatter", "RawTokenFormatter", "TestcaseFormatter"]

class NullFormatter(Formatter[_T]):
    def format(self, tokensource: Iterable[tuple[_TokenType, str]], outfile: SupportsWrite[_T]) -> None: ...

class RawTokenFormatter(Formatter[bytes]):
    encoding: str
    compress: Incomplete
    error_color: Incomplete
    def format(self, tokensource: Iterable[tuple[_TokenType, str]], outfile: SupportsWrite[bytes]) -> None: ...

class TestcaseFormatter(Formatter[_T]):
    def format(self, tokensource: Iterable[tuple[_TokenType, str]], outfile: SupportsWrite[_T]) -> None: ...



from _typeshed import Incomplete, Unused
from typing_extensions import Self

class ControlClientError(Exception): ...

class ControlClient:
    socket_path: str
    timeout: float
    def __init__(self, socket_path: str, timeout: float = 30.0) -> None: ...
    def connect(self) -> None: ...
    def close(self) -> None: ...
    def send_command(self, command: str, args: list[str] | None = None) -> dict[Incomplete, Incomplete]: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: Unused) -> None: ...

def parse_command(line: str) -> tuple[str, list[str]]: ...



from _typeshed import Incomplete

class ADPersistentSearch:
    connection: Incomplete
    message_id: Incomplete
    base: Incomplete
    scope: Incomplete
    attributes: Incomplete
    controls: Incomplete
    filter: str
    def __init__(self, connection, search_base, search_scope, attributes, streaming, callback) -> None: ...
    def start(self) -> None: ...
    def stop(self, unbind: bool = True) -> None: ...
    def next(self, block: bool = False, timeout=None): ...
    def funnel(self, block: bool = False, timeout=None) -> None: ...



from _typeshed import Incomplete
from typing_extensions import deprecated

from braintree.error_result import ErrorResult
from braintree.successful_result import SuccessfulResult

class MerchantGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    @deprecated("gateway.merchant.create(...) is deprecated and will be removed in a future version.")
    def create(self, params: dict[str, Incomplete] | None) -> SuccessfulResult | ErrorResult: ...



import json
from _typeshed import SupportsItems, SupportsKeysAndGetItem
from _typeshed.wsgi import WSGIApplication, WSGIEnvironment
from collections.abc import Iterable, Sequence
from http.cookiejar import CookieJar, DefaultCookiePolicy
from typing import Any, Generic, Literal, TypeAlias, TypeVar

from webob.request import BaseRequest
from webtest.forms import File, Upload
from webtest.response import TestResponse

# NOTE: While it is possible to pass different kinds of values depending on
#       the exact configuration of the request, it seems more robust to
#       restrict them to the types that are supported by all code paths.
#       I don't expect anyone to try to pass different kinds of values
#       in a non-JSON request.
_ParamValue: TypeAlias = File | Upload | int | bytes | str
_Params: TypeAlias = SupportsItems[str | bytes, _ParamValue] | Sequence[tuple[str | bytes, _ParamValue]]
# NOTE: Using `Collection` rather than `Iterable` would probably be slightly
#       safer since WebTest will check this parameter for truthyness. But since
#       objects are truthy by default, this should only lead to issues in truly
#       exotic cases.
_ExtraEnviron: TypeAlias = SupportsKeysAndGetItem[str, Any] | Iterable[tuple[str, Any]]
_Files: TypeAlias = Sequence[tuple[str, str] | tuple[str, str, bytes]]
_AppT = TypeVar("_AppT", bound=WSGIApplication, default=WSGIApplication)

__all__ = ["TestApp", "TestRequest"]

class AppError(Exception):
    def __init__(self, message: str, *args: object) -> None: ...

class CookiePolicy(DefaultCookiePolicy): ...

class TestRequest(BaseRequest):
    ResponseClass: type[TestResponse]
    __test__: Literal[False]

class TestApp(Generic[_AppT]):
    RequestClass: type[TestRequest]
    app: _AppT
    lint: bool
    relative_to: str | None
    extra_environ: WSGIEnvironment
    use_unicode: bool
    cookiejar: CookieJar
    JSONEncoder: json.JSONEncoder
    __test__: Literal[False]
    def __init__(
        self,
        app: _AppT,
        # NOTE: this extra_environ is different from the others and needs to
        #       support __delitem__, it seems easiest to just treat this like
        #       a regular WSGIEnvironment. The docs also say that this should
        #       be a dictionary.
        extra_environ: WSGIEnvironment | None = None,
        relative_to: str | None = None,
        use_unicode: bool = True,
        cookiejar: CookieJar | None = None,
        parser_features: Sequence[str] | str | None = None,
        json_encoder: json.JSONEncoder | None = None,
        lint: bool = True,
    ) -> None: ...
    def get_authorization(self) -> tuple[str, str | tuple[str, str]]: ...
    def set_authorization(self, value: tuple[str, str | tuple[str, str]]) -> None: ...

    @property
    def authorization(self) -> tuple[str, str | tuple[str, str]]: ...
    @authorization.setter
    def authorization(self, value: tuple[str, str | tuple[str, str]]) -> None: ...

    @property
    def cookies(self) -> dict[str, str | None]: ...
    def set_cookie(self, name: str, value: str | None) -> None: ...
    def reset(self) -> None: ...
    def set_parser_features(self, parser_features: Sequence[str] | str) -> None: ...
    def get(
        self,
        url: str,
        params: _Params | str | None = None,
        headers: dict[str, str] | None = None,
        extra_environ: _ExtraEnviron | None = None,
        status: int | str | None = None,
        expect_errors: bool = False,
        xhr: bool = False,
    ) -> TestResponse: ...
    def post(
        self,
        url: str,
        params: _Params | str = "",
        headers: dict[str, str] | None = None,
        extra_environ: _ExtraEnviron | None = None,
        status: int | str | None = None,
        upload_files: _Files | None = None,
        expect_errors: bool = False,
        content_type: str | None = None,
        xhr: bool = False,
    ) -> TestResponse: ...
    def put(
        self,
        url: str,
        params: _Params | str = "",
        headers: dict[str, str] | None = None,
        extra_environ: _ExtraEnviron | None = None,
        status: int | str | None = None,
        upload_files: _Files | None = None,
        expect_errors: bool = False,
        content_type: str | None = None,
        xhr: bool = False,
    ) -> TestResponse: ...
    def patch(
        self,
        url: str,
        params: _Params | str = "",
        headers: dict[str, str] | None = None,
        extra_environ: _ExtraEnviron | None = None,
        status: int | str | None = None,
        upload_files: _Files | None = None,
        expect_errors: bool = False,
        content_type: str | None = None,
        xhr: bool = False,
    ) -> TestResponse: ...
    def delete(
        self,
        url: str,
        params: _Params | str = "",
        headers: dict[str, str] | None = None,
        extra_environ: _ExtraEnviron | None = None,
        status: int | str | None = None,
        expect_errors: bool = False,
        content_type: str | None = None,
        xhr: bool = False,
    ) -> TestResponse: ...
    def options(
        self,
        url: str,
        headers: dict[str, str] | None = None,
        extra_environ: _ExtraEnviron | None = None,
        status: int | str | None = None,
        expect_errors: bool = False,
        xhr: bool = False,
    ) -> TestResponse: ...
    def head(
        self,
        url: str,
        params: _Params | str | None = None,
        headers: dict[str, str] | None = None,
        extra_environ: _ExtraEnviron | None = None,
        status: int | str | None = None,
        expect_errors: bool = False,
        xhr: bool = False,
    ) -> TestResponse: ...
    def post_json(
        self,
        url: str,
        params: Any = ...,
        *,
        headers: dict[str, str] | None = None,
        extra_environ: _ExtraEnviron | None = None,
        status: int | str | None = None,
        expect_errors: bool = False,
        content_type: str | None = None,
        xhr: bool = False,
    ) -> TestResponse: ...
    def put_json(
        self,
        url: str,
        params: Any = ...,
        *,
        headers: dict[str, str] | None = None,
        extra_environ: _ExtraEnviron | None = None,
        status: int | str | None = None,
        expect_errors: bool = False,
        content_type: str | None = None,
        xhr: bool = False,
    ) -> TestResponse: ...
    def patch_json(
        self,
        url: str,
        params: Any = ...,
        *,
        headers: dict[str, str] | None = None,
        extra_environ: _ExtraEnviron | None = None,
        status: int | str | None = None,
        expect_errors: bool = False,
        content_type: str | None = None,
        xhr: bool = False,
    ) -> TestResponse: ...
    def delete_json(
        self,
        url: str,
        params: Any = ...,
        *,
        headers: dict[str, str] | None = None,
        extra_environ: _ExtraEnviron | None = None,
        status: int | str | None = None,
        expect_errors: bool = False,
        content_type: str | None = None,
        xhr: bool = False,
    ) -> TestResponse: ...
    def encode_multipart(self, params: Iterable[tuple[str | bytes, _ParamValue]], files: _Files) -> tuple[str, bytes]: ...
    def request(
        self, url_or_req: str | TestRequest, status: int | str | None = None, expect_errors: bool = False, **req_params: Any
    ) -> TestResponse: ...
    def do_request(
        self, req: TestRequest, status: int | str | None = None, expect_errors: bool | None = None
    ) -> TestResponse: ...



from _typeshed import Incomplete
from enum import IntEnum

log: Incomplete

class FinalState(IntEnum):
    ABORTED = 0
    FAILED = 1
    SUCCEEDED = 2

class ProducerBatch:
    max_record_size: int
    created: Incomplete
    drained: Incomplete
    attempts: int
    last_attempt: Incomplete
    last_append: Incomplete
    records: Incomplete
    topic_partition: Incomplete
    produce_future: Incomplete
    def __init__(self, tp, records, now=None) -> None: ...
    @property
    def final_state(self): ...
    @property
    def record_count(self): ...
    @property
    def producer_id(self): ...
    @property
    def producer_epoch(self): ...
    @property
    def has_sequence(self): ...
    def try_append(self, timestamp_ms, key, value, headers, now=None): ...
    def abort(self, exception): ...
    def complete(self, base_offset, log_append_time): ...
    def complete_exceptionally(self, top_level_exception, record_exceptions_fn): ...
    def done(self, base_offset=None, timestamp_ms=None, top_level_exception=None, record_exceptions_fn=None): ...
    def has_reached_delivery_timeout(self, delivery_timeout_ms, now=None): ...
    def in_retry(self): ...
    def retry(self, now=None) -> None: ...
    @property
    def is_done(self): ...
    def __lt__(self, other): ...



import re
from collections.abc import Iterable, Sequence
from operator import itemgetter
from typing import Final

CS_ESCAPE: Final[re.Pattern[str]]
FIRST_ELEMENT: Final[itemgetter[int]]

def commonprefix(m: Iterable[str]) -> str: ...
def make_charset(letters: Iterable[str]) -> str: ...
def regex_opt_inner(strings: Sequence[str], open_paren: str) -> str: ...
def regex_opt(strings: Iterable[str], prefix: str = "", suffix: str = "") -> str: ...



from _typeshed import Incomplete

log: Incomplete

class Future:
    error_on_callbacks: bool
    is_done: bool
    value: Incomplete
    exception: Incomplete
    def __init__(self) -> None: ...
    def succeeded(self): ...
    def failed(self): ...
    def retriable(self): ...
    def success(self, value): ...
    def failure(self, e): ...
    def add_callback(self, f, *args, **kwargs): ...
    def add_errback(self, f, *args, **kwargs): ...
    def add_both(self, f, *args, **kwargs): ...
    def chain(self, future): ...



from _typeshed import Incomplete

from kafka.protocol.abstract import AbstractType

class Struct(AbstractType):
    SCHEMA: Incomplete
    def __init__(self, *args, **kwargs) -> None: ...
    @classmethod
    def encode(cls, item): ...
    @classmethod
    def decode(cls, data): ...
    def get_item(self, name): ...
    def __hash__(self): ...
    def __eq__(self, other): ...



import types
from _typeshed import SupportsWrite
from collections.abc import Iterable, Sequence
from typing import Any, ClassVar, Generic, TypeVar, overload

from pygments.style import Style
from pygments.token import _TokenType

_T = TypeVar("_T", str, bytes)

__all__ = ["Formatter"]

class Formatter(Generic[_T]):
    name: ClassVar[str]  # Set to None, but always overridden with a non-None value in subclasses.
    aliases: ClassVar[Sequence[str]]  # Not intended to be mutable
    filenames: ClassVar[Sequence[str]]  # Not intended to be mutable
    unicodeoutput: ClassVar[bool]
    style: type[Style]
    full: bool
    title: str
    encoding: str | None
    options: dict[str, Any]  # arbitrary values used by subclasses

    @overload
    def __init__(
        self: Formatter[str],
        *,
        style: type[Style] | str = "default",
        full: bool = False,
        title: str = "",
        encoding: None = None,
        outencoding: None = None,
        **options: Any,  # arbitrary values used by subclasses
    ) -> None: ...
    @overload
    def __init__(
        self: Formatter[bytes],
        *,
        style: type[Style] | str = "default",
        full: bool = False,
        title: str = "",
        encoding: str,
        outencoding: None = None,
        **options: Any,  # arbitrary values used by subclasses
    ) -> None: ...
    @overload
    def __init__(
        self: Formatter[bytes],
        *,
        style: type[Style] | str = "default",
        full: bool = False,
        title: str = "",
        encoding: None = None,
        outencoding: str,
        **options: Any,  # arbitrary values used by subclasses
    ) -> None: ...

    def __class_getitem__(cls, name: Any) -> types.GenericAlias: ...
    def get_style_defs(self, arg: str = "") -> str: ...
    def format(self, tokensource: Iterable[tuple[_TokenType, str]], outfile: SupportsWrite[_T]) -> None: ...



import abc
from _typeshed import Incomplete

from kafka.sasl.abc import SaslMechanism

log: Incomplete

class SaslMechanismOAuth(SaslMechanism):
    token_provider: Incomplete
    def __init__(self, **config) -> None: ...
    def auth_bytes(self): ...
    def receive(self, auth_bytes) -> None: ...
    def is_done(self): ...
    def is_authenticated(self): ...
    def auth_details(self): ...

ABC: Incomplete

class AbstractTokenProvider(ABC, metaclass=abc.ABCMeta):
    def __init__(self, **config) -> None: ...
    @abc.abstractmethod
    def token(self): ...
    def extensions(self): ...



from typing import Final

UNICODE_ASCII_CHARACTER_SET: Final[str]

def generate_token(length: int = 30, chars: str = UNICODE_ASCII_CHARACTER_SET) -> str: ...  # noqa: Y011
def is_secure_transport(uri: str) -> bool: ...



import logging
from _typeshed import Incomplete

from authlib.oauth1 import AuthorizationServer as _AuthorizationServer, OAuth1Request, TemporaryCredential

log: logging.Logger

class BaseServer(_AuthorizationServer):
    token_generator: Incomplete
    client_model: Incomplete
    token_model: Incomplete
    SUPPORTED_SIGNATURE_METHODS: Incomplete
    def __init__(self, client_model, token_model, token_generator=None) -> None: ...
    def get_client_by_id(self, client_id): ...
    def exists_nonce(self, nonce, request) -> bool: ...
    def create_token_credential(self, request): ...
    def check_authorization_request(self, request) -> OAuth1Request: ...
    def create_oauth1_request(self, request) -> OAuth1Request: ...
    def handle_response(self, status_code, payload, headers): ...

class CacheAuthorizationServer(BaseServer):
    def __init__(self, client_model, token_model, token_generator=None) -> None: ...
    def create_temporary_credential(self, request) -> TemporaryCredential: ...
    def get_temporary_credential(self, request) -> TemporaryCredential | None: ...
    def delete_temporary_credential(self, request) -> None: ...
    def create_authorization_verifier(self, request) -> str: ...



import abc

class SaslMechanism(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __init__(self, **config): ...
    @abc.abstractmethod
    def auth_bytes(self): ...
    @abc.abstractmethod
    def receive(self, auth_bytes): ...
    @abc.abstractmethod
    def is_done(self): ...
    @abc.abstractmethod
    def is_authenticated(self): ...
    def auth_details(self): ...



class Subscription:
    def __init__(self, metadata, group_instance_id) -> None: ...
    @property
    def version(self): ...
    @property
    def user_data(self): ...
    @property
    def topics(self): ...
    subscription = topics
    @property
    def group_instance_id(self): ...
    def encode(self): ...
    def __eq__(self, other): ...



from _typeshed import Incomplete

log: Incomplete

class AtomicInteger:
    def __init__(self, val: int = 0) -> None: ...
    def increment(self): ...
    def decrement(self): ...
    def get(self): ...

class RecordAccumulator:
    DEFAULT_CONFIG: Incomplete
    config: Incomplete
    muted: Incomplete
    def __init__(self, **configs) -> None: ...
    @property
    def delivery_timeout_ms(self): ...
    @property
    def next_expiry_time_ms(self): ...
    def append(self, tp, timestamp_ms, key, value, headers, now=None): ...
    def reset_next_batch_expiry_time(self) -> None: ...
    def maybe_update_next_batch_expiry_time(self, batch) -> None: ...
    def expired_batches(self, now=None): ...
    def reenqueue(self, batch, now=None) -> None: ...
    def ready(self, cluster, now=None): ...
    def has_undrained(self): ...
    def drain_batches_for_one_node(self, cluster, node_id, max_size, now=None): ...
    def drain(self, cluster, nodes, max_size, now=None): ...
    def deallocate(self, batch) -> None: ...
    def flush_in_progress(self): ...
    def begin_flush(self) -> None: ...
    def await_flush_completion(self, timeout=None) -> None: ...
    @property
    def has_incomplete(self): ...
    def abort_incomplete_batches(self) -> None: ...
    def abort_undrained_batches(self, error) -> None: ...
    def close(self) -> None: ...

class IncompleteProducerBatches:
    def __init__(self) -> None: ...
    def add(self, batch) -> None: ...
    def remove(self, batch) -> None: ...
    def all(self): ...
    def __bool__(self) -> bool: ...
    __nonzero__ = __bool__



from _typeshed import Incomplete
from asyncio import AbstractEventLoop, Future, Handle
from collections.abc import Callable, Sequence
from logging import Logger
from typing_extensions import Self

from ..connection import Connection, Parameters
from .base_connection import BaseConnection
from .utils import io_services_utils
from .utils.connection_workflow import AbstractAMQPConnectionWorkflow, AMQPConnectorException
from .utils.nbio_interface import AbstractFileDescriptorServices, AbstractIOReference, AbstractIOServices, AbstractTimerReference

LOGGER: Logger

class AsyncioConnection(BaseConnection):
    def __init__(
        self,
        parameters: Parameters | None = None,
        on_open_callback: Callable[[Self], object] | None = None,
        on_open_error_callback: Callable[[Self, BaseException], object] | None = None,
        on_close_callback: Callable[[Self, BaseException], object] | None = None,
        custom_ioloop: AbstractEventLoop | None = None,
        internal_connection_workflow: bool = True,
    ) -> None: ...
    @classmethod
    def create_connection(
        cls,
        connection_configs: Sequence[Parameters],
        on_done: Callable[[Connection | AMQPConnectorException], object],
        custom_ioloop: AbstractEventLoop | None = None,
        workflow: AbstractAMQPConnectionWorkflow | None = None,
    ) -> AbstractAMQPConnectionWorkflow: ...

class _AsyncioIOServicesAdapter(
    io_services_utils.SocketConnectionMixin,
    io_services_utils.StreamingConnectionMixin,
    AbstractIOServices,
    AbstractFileDescriptorServices,
):
    def __init__(self, loop: AbstractEventLoop | None = None) -> None: ...
    def get_native_ioloop(self) -> AbstractEventLoop: ...
    def close(self) -> None: ...
    def run(self) -> None: ...
    def stop(self) -> None: ...
    def add_callback_threadsafe(self, callback: Callable[[], object]) -> None: ...
    def call_later(self, delay: float, callback: Callable[[], object]) -> _TimerHandle: ...
    def getaddrinfo(
        self,
        host: str,
        port: int,
        on_done: Callable[..., object],
        family: int = 0,
        socktype: int = 0,
        proto: int = 0,
        flags: int = 0,
    ) -> AbstractIOReference: ...
    def set_reader(self, fd: int, on_readable: Callable[[], object]) -> None: ...
    def remove_reader(self, fd: int) -> bool: ...
    def set_writer(self, fd: int, on_writable: Callable[[], object]) -> None: ...
    def remove_writer(self, fd: int) -> bool: ...

class _TimerHandle(AbstractTimerReference):
    def __init__(self, handle: Handle) -> None: ...
    def cancel(self) -> None: ...

class _AsyncioIOReference(AbstractIOReference):
    def __init__(self, future: Future[Incomplete], on_done: Callable[[BaseConnection | BaseException], object]) -> None: ...
    def cancel(self) -> bool: ...



from _typeshed import Incomplete

from kafka.protocol.api import Request, Response

class InitProducerIdResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class InitProducerIdResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class InitProducerIdRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = InitProducerIdResponse_v0
    SCHEMA: Incomplete

class InitProducerIdRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = InitProducerIdResponse_v1
    SCHEMA: Incomplete

InitProducerIdRequest: Incomplete
InitProducerIdResponse: Incomplete



import abc
from _typeshed import Incomplete
from enum import IntEnum

from kafka.util import synchronized

log: Incomplete

class SubscriptionType(IntEnum):
    NONE = 0
    AUTO_TOPICS = 1
    AUTO_PATTERN = 2
    USER_ASSIGNED = 3

class SubscriptionState:
    subscription: Incomplete
    subscription_type: Incomplete
    subscribed_pattern: Incomplete
    assignment: Incomplete
    rebalance_listener: Incomplete
    listeners: Incomplete
    def __init__(self, offset_reset_strategy: str = "earliest") -> None: ...
    @synchronized
    def subscribe(self, topics=(), pattern=None, listener=None) -> None: ...
    @synchronized
    def change_subscription(self, topics) -> None: ...
    @synchronized
    def group_subscribe(self, topics) -> None: ...
    @synchronized
    def reset_group_subscription(self) -> None: ...
    @synchronized
    def assign_from_user(self, partitions) -> None: ...
    @synchronized
    def assign_from_subscribed(self, assignments) -> None: ...
    @synchronized
    def unsubscribe(self) -> None: ...
    @synchronized
    def group_subscription(self): ...
    @synchronized
    def seek(self, partition, offset) -> None: ...
    @synchronized
    def assigned_partitions(self): ...
    @synchronized
    def paused_partitions(self): ...
    @synchronized
    def fetchable_partitions(self): ...
    @synchronized
    def partitions_auto_assigned(self): ...
    @synchronized
    def all_consumed_offsets(self): ...
    @synchronized
    def request_offset_reset(self, partition, offset_reset_strategy=None) -> None: ...
    @synchronized
    def set_reset_pending(self, partitions, next_allowed_reset_time) -> None: ...
    @synchronized
    def has_default_offset_reset_policy(self): ...
    @synchronized
    def is_offset_reset_needed(self, partition): ...
    @synchronized
    def has_all_fetch_positions(self): ...
    @synchronized
    def missing_fetch_positions(self): ...
    @synchronized
    def has_valid_position(self, partition): ...
    @synchronized
    def reset_missing_positions(self) -> None: ...
    @synchronized
    def partitions_needing_reset(self): ...
    @synchronized
    def is_assigned(self, partition): ...
    @synchronized
    def is_paused(self, partition): ...
    @synchronized
    def is_fetchable(self, partition): ...
    @synchronized
    def pause(self, partition) -> None: ...
    @synchronized
    def resume(self, partition) -> None: ...
    @synchronized
    def reset_failed(self, partitions, next_retry_time) -> None: ...
    @synchronized
    def move_partition_to_end(self, partition) -> None: ...
    @synchronized
    def position(self, partition): ...

class TopicPartitionState:
    paused: bool
    reset_strategy: Incomplete
    highwater: Incomplete
    drop_pending_record_batch: bool
    next_allowed_retry_time: Incomplete
    def __init__(self) -> None: ...
    position: Incomplete
    def reset(self, strategy) -> None: ...
    def is_reset_allowed(self): ...
    @property
    def awaiting_reset(self): ...
    def set_reset_pending(self, next_allowed_retry_time) -> None: ...
    def reset_failed(self, next_allowed_retry_time) -> None: ...
    @property
    def has_valid_position(self): ...
    def is_missing_position(self): ...
    def seek(self, offset) -> None: ...
    def pause(self) -> None: ...
    def resume(self) -> None: ...
    def is_fetchable(self): ...

class ConsumerRebalanceListener(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def on_partitions_revoked(self, revoked): ...
    @abc.abstractmethod
    def on_partitions_assigned(self, assigned): ...



from collections.abc import Iterable

from .ast import Node

def serialize(nodes: Iterable[Node]) -> str: ...
def serialize_identifier(value: str) -> str: ...
def serialize_name(value: str) -> str: ...
def serialize_string_value(value: str) -> str: ...
def serialize_url(value: str) -> str: ...

BAD_PAIRS: set[tuple[str, str]]



import abc
from _typeshed import Incomplete

log: Incomplete

class AbstractPartitionAssignor(metaclass=abc.ABCMeta):
    @property
    @abc.abstractmethod
    def name(self): ...
    @abc.abstractmethod
    def assign(self, cluster, members): ...
    @abc.abstractmethod
    def metadata(self, topics): ...
    @abc.abstractmethod
    def on_assignment(self, assignment): ...



from gunicorn.uwsgi.errors import (
    ForbiddenUWSGIRequest as ForbiddenUWSGIRequest,
    InvalidUWSGIHeader as InvalidUWSGIHeader,
    UnsupportedModifier as UnsupportedModifier,
    UWSGIParseException as UWSGIParseException,
)
from gunicorn.uwsgi.message import UWSGIRequest as UWSGIRequest
from gunicorn.uwsgi.parser import UWSGIParser as UWSGIParser

__all__ = [
    "UWSGIRequest",
    "UWSGIParser",
    "UWSGIParseException",
    "InvalidUWSGIHeader",
    "UnsupportedModifier",
    "ForbiddenUWSGIRequest",
]



from typing import TypeVar

from google.protobuf.any_pb2 import Any
from google.protobuf.descriptor import Descriptor
from google.protobuf.message import Message

_MessageT = TypeVar("_MessageT", bound=Message)

def pack(msg: Message, type_url_prefix: str | None = "type.googleapis.com/", deterministic: bool | None = None) -> Any: ...
def unpack(any_msg: Any, msg: Message) -> bool: ...
def unpack_as(any_msg: Any, message_type: type[_MessageT]) -> _MessageT: ...
def type_name(any_msg: Any) -> str: ...
def is_type(any_msg: Any, des: Descriptor) -> bool: ...



import asyncio
from _typeshed import Incomplete
from asyncio import StreamReader, StreamWriter
from signal import Signals
from typing import ClassVar

from gunicorn.config import Config
from gunicorn.dirty.worker import DirtyWorker
from gunicorn.glogging import Logger as GLogger

class DirtyArbiter:
    SIGNALS: ClassVar[list[Signals]]
    WORKER_BOOT_ERROR: ClassVar[int]
    cfg: Config
    log: GLogger
    pid: int | None
    ppid: int
    pidfile: str | None
    tmpdir: str
    socket_path: str
    workers: dict[int, DirtyWorker]
    worker_sockets: dict[int, str]
    worker_connections: dict[int, tuple[Incomplete, Incomplete]]
    worker_queues: dict[int, asyncio.Queue[Incomplete]]
    worker_consumers: dict[int, asyncio.Task[None]]
    worker_age: int
    alive: bool
    num_workers: int
    app_specs: dict[str, dict[Incomplete, Incomplete]]
    app_worker_map: dict[str, set[Incomplete]]
    worker_app_map: dict[int, list[Incomplete]]
    stash_tables: dict[str, dict[Incomplete, Incomplete]]

    def __init__(self, cfg: Config, log: GLogger, socket_path: str | None = None, pidfile: str | None = None) -> None: ...
    def run(self) -> None: ...
    def init_signals(self) -> None: ...
    async def handle_client(self, reader: StreamReader, writer: StreamWriter) -> None: ...
    async def route_request(self, request: dict[str, Incomplete], client_writer: StreamWriter) -> None: ...
    async def handle_status_request(self, message: dict[str, Incomplete], client_writer: StreamWriter) -> None: ...
    async def handle_manage_request(self, message: dict[str, Incomplete], client_writer: StreamWriter) -> None: ...
    async def handle_stash_request(self, message: dict[str, Incomplete], client_writer: StreamWriter) -> None: ...
    async def manage_workers(self) -> None: ...
    def spawn_worker(self, force_all_apps: bool = False) -> int | None: ...
    def kill_worker(self, pid: int, sig: int) -> None: ...
    async def murder_workers(self) -> None: ...
    def reap_workers(self) -> None: ...
    async def reload(self) -> None: ...
    async def stop(self, graceful: bool = True) -> None: ...



from collections.abc import Callable, Iterable
from enum import IntEnum
from typing import Any, Final, Literal, SupportsIndex, TypeAlias, TypedDict, type_check_only
from typing_extensions import Self

_H1CProtocol: TypeAlias = Any  # gunicorn_h1c H1CProtocol class

class ParseError(Exception): ...
class InvalidProxyLine(ParseError): ...
class InvalidProxyHeader(ParseError): ...

PP_V2_SIGNATURE: Final[bytes]
RFC9110_6_5_1_FORBIDDEN_TRAILER: Final[frozenset[bytes]]

class PPCommand(IntEnum):
    LOCAL = 0x0
    PROXY = 0x1

class PPFamily(IntEnum):
    UNSPEC = 0x0
    INET = 0x1
    INET6 = 0x2
    UNIX = 0x3

class PPProtocol(IntEnum):
    UNSPEC = 0x0
    STREAM = 0x1
    DGRAM = 0x2

class LimitRequestLine(ParseError): ...
class InvalidRequestLine(ParseError): ...
class LimitRequestHeaders(ParseError): ...
class InvalidRequestMethod(ParseError): ...
class InvalidHTTPVersion(ParseError): ...
class InvalidHeaderName(ParseError): ...
class InvalidHeader(ParseError): ...
class UnsupportedTransferCoding(ParseError): ...
class InvalidChunkSize(ParseError): ...
class InvalidChunkExtension(ParseError): ...

@type_check_only
class _ProxyProtocolInfo(TypedDict):
    proxy_protocol: Literal["TCP4", "TCP6", "UDP4", "UDP6"]
    client_addr: str
    client_port: int
    proxy_addr: str
    proxy_port: int

@type_check_only
class _ProxyProtocolInfoUnknown(TypedDict):
    proxy_protocol: Literal["UNKNOWN", "LOCAL", "UNSPEC"]
    client_addr: None
    client_port: None
    proxy_addr: None
    proxy_port: None

class PythonProtocol:
    __slots__ = (
        "_on_message_begin",
        "_on_url",
        "_on_header",
        "_on_headers_complete",
        "_on_body",
        "_on_message_complete",
        "_state",
        "_buffer",
        "_headers_list",
        "method",
        "path",
        "http_version",
        "headers",
        "content_length",
        "is_chunked",
        "should_keep_alive",
        "is_complete",
        "_body_remaining",
        "_skip_body",
        "_chunk_state",
        "_chunk_size",
        "_chunk_remaining",
        "_limit_request_line",
        "_limit_request_fields",
        "_limit_request_field_size",
        "_permit_unconventional_http_method",
        "_permit_unconventional_http_version",
        "_header_count",
        "_proxy_protocol",
        "_proxy_protocol_info",
        "_proxy_protocol_done",
    )
    method: bytes | None
    path: bytes | None
    http_version: tuple[int, int] | None
    headers: list[tuple[bytes, bytes]]
    content_length: int | None
    is_chunked: bool
    should_keep_alive: bool
    is_complete: bool

    def __init__(
        self,
        on_message_begin: Callable[[], object] | None = None,
        on_url: Callable[[bytes], object] | None = None,
        on_header: Callable[[bytes, bytes], object] | None = None,
        on_headers_complete: Callable[[], bool] | None = None,
        on_body: Callable[[bytes], object] | None = None,
        on_message_complete: Callable[[], object] | None = None,
        limit_request_line: int = 8190,
        limit_request_fields: int = 100,
        limit_request_field_size: int = 8190,
        permit_unconventional_http_method: bool = False,
        permit_unconventional_http_version: bool = False,
        proxy_protocol: Literal["off", "v1", "v2", "auto"] = "off",
    ) -> None: ...
    def feed(self, data: Iterable[SupportsIndex]) -> None: ...
    @property
    def proxy_protocol_info(self) -> _ProxyProtocolInfo | _ProxyProtocolInfoUnknown | None: ...
    def reset(self) -> None: ...
    def finish(self) -> None: ...

class CallbackRequest:
    __slots__ = (
        "method",
        "uri",
        "path",
        "query",
        "fragment",
        "version",
        "headers",
        "headers_bytes",
        "scheme",
        "raw_path",
        "content_length",
        "chunked",
        "must_close",
        "proxy_protocol_info",
        "_expect_100_continue",
    )
    method: str | None
    uri: str | None
    path: str | None
    query: str | None
    fragment: str | None
    version: tuple[int, int] | None
    headers: list[tuple[str, str]]
    headers_bytes: list[tuple[bytes, bytes]]
    scheme: Literal["https", "http"]
    raw_path: bytes
    content_length: int
    chunked: bool
    must_close: bool
    proxy_protocol_info: dict[str, str | int | None] | None  # TODO: Use TypedDict

    def __init__(self) -> None: ...
    @classmethod
    def from_parser(cls, parser: _H1CProtocol | PythonProtocol, is_ssl: bool = False) -> Self: ...
    def should_close(self) -> bool: ...
    def get_header(self, name: str) -> str | None: ...



from _typeshed import Incomplete
from enum import IntEnum

from kafka.protocol.api import Request, Response

class CreateTopicsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class CreateTopicsResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class CreateTopicsResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class CreateTopicsResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class CreateTopicsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = CreateTopicsResponse_v0
    SCHEMA: Incomplete

class CreateTopicsRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = CreateTopicsResponse_v1
    SCHEMA: Incomplete

class CreateTopicsRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = CreateTopicsResponse_v2
    SCHEMA: Incomplete

class CreateTopicsRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = CreateTopicsResponse_v3
    SCHEMA: Incomplete

CreateTopicsRequest: Incomplete
CreateTopicsResponse: Incomplete

class DeleteTopicsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DeleteTopicsResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DeleteTopicsResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DeleteTopicsResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DeleteTopicsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DeleteTopicsResponse_v0
    SCHEMA: Incomplete

class DeleteTopicsRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DeleteTopicsResponse_v1
    SCHEMA: Incomplete

class DeleteTopicsRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DeleteTopicsResponse_v2
    SCHEMA: Incomplete

class DeleteTopicsRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DeleteTopicsResponse_v3
    SCHEMA: Incomplete

DeleteTopicsRequest: Incomplete
DeleteTopicsResponse: Incomplete

class DeleteRecordsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DeleteRecordsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DeleteRecordsResponse_v0
    SCHEMA: Incomplete

DeleteRecordsResponse: Incomplete
DeleteRecordsRequest: Incomplete

class ListGroupsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ListGroupsResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ListGroupsResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ListGroupsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ListGroupsResponse_v0
    SCHEMA: Incomplete

class ListGroupsRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ListGroupsResponse_v1
    SCHEMA: Incomplete

class ListGroupsRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ListGroupsResponse_v2
    SCHEMA: Incomplete

ListGroupsRequest: Incomplete
ListGroupsResponse: Incomplete

class DescribeGroupsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DescribeGroupsResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DescribeGroupsResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DescribeGroupsResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DescribeGroupsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DescribeGroupsResponse_v0
    SCHEMA: Incomplete

class DescribeGroupsRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DescribeGroupsResponse_v1
    SCHEMA: Incomplete

class DescribeGroupsRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DescribeGroupsResponse_v2
    SCHEMA: Incomplete

class DescribeGroupsRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DescribeGroupsResponse_v3
    SCHEMA: Incomplete

DescribeGroupsRequest: Incomplete
DescribeGroupsResponse: Incomplete

class DescribeAclsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DescribeAclsResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DescribeAclsResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DescribeAclsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DescribeAclsResponse_v0
    SCHEMA: Incomplete

class DescribeAclsRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DescribeAclsResponse_v1
    SCHEMA: Incomplete

class DescribeAclsRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DescribeAclsResponse_v2
    SCHEMA: Incomplete

DescribeAclsRequest: Incomplete
DescribeAclsResponse: Incomplete

class CreateAclsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class CreateAclsResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class CreateAclsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = CreateAclsResponse_v0
    SCHEMA: Incomplete

class CreateAclsRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = CreateAclsResponse_v1
    SCHEMA: Incomplete

CreateAclsRequest: Incomplete
CreateAclsResponse: Incomplete

class DeleteAclsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DeleteAclsResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DeleteAclsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DeleteAclsResponse_v0
    SCHEMA: Incomplete

class DeleteAclsRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DeleteAclsResponse_v1
    SCHEMA: Incomplete

DeleteAclsRequest: Incomplete
DeleteAclsResponse: Incomplete

class AlterConfigsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class AlterConfigsResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class AlterConfigsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = AlterConfigsResponse_v0
    SCHEMA: Incomplete

class AlterConfigsRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = AlterConfigsResponse_v1
    SCHEMA: Incomplete

AlterConfigsRequest: Incomplete
AlterConfigsResponse: Incomplete

class DescribeConfigsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DescribeConfigsResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DescribeConfigsResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DescribeConfigsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DescribeConfigsResponse_v0
    SCHEMA: Incomplete

class DescribeConfigsRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DescribeConfigsResponse_v1
    SCHEMA: Incomplete

class DescribeConfigsRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DescribeConfigsResponse_v2
    SCHEMA: Incomplete

DescribeConfigsRequest: Incomplete
DescribeConfigsResponse: Incomplete

class DescribeLogDirsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DescribeLogDirsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DescribeLogDirsResponse_v0
    SCHEMA: Incomplete

DescribeLogDirsResponse: Incomplete
DescribeLogDirsRequest: Incomplete

class SaslAuthenticateResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class SaslAuthenticateResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class SaslAuthenticateRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = SaslAuthenticateResponse_v0
    SCHEMA: Incomplete

class SaslAuthenticateRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = SaslAuthenticateResponse_v1
    SCHEMA: Incomplete

SaslAuthenticateRequest: Incomplete
SaslAuthenticateResponse: Incomplete

class CreatePartitionsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class CreatePartitionsResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class CreatePartitionsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = CreatePartitionsResponse_v0
    SCHEMA: Incomplete

class CreatePartitionsRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete
    RESPONSE_TYPE = CreatePartitionsResponse_v1

CreatePartitionsRequest: Incomplete
CreatePartitionsResponse: Incomplete

class DeleteGroupsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DeleteGroupsResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DeleteGroupsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DeleteGroupsResponse_v0
    SCHEMA: Incomplete

class DeleteGroupsRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DeleteGroupsResponse_v1
    SCHEMA: Incomplete

DeleteGroupsRequest: Incomplete
DeleteGroupsResponse: Incomplete

class DescribeClientQuotasResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class DescribeClientQuotasRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = DescribeClientQuotasResponse_v0
    SCHEMA: Incomplete

DescribeClientQuotasRequest: Incomplete
DescribeClientQuotasResponse: Incomplete

class AlterPartitionReassignmentsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete
    FLEXIBLE_VERSION: bool

class AlterPartitionReassignmentsRequest_v0(Request):
    FLEXIBLE_VERSION: bool
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = AlterPartitionReassignmentsResponse_v0
    SCHEMA: Incomplete

AlterPartitionReassignmentsRequest: Incomplete
AlterPartitionReassignmentsResponse: Incomplete

class ListPartitionReassignmentsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete
    FLEXIBLE_VERSION: bool

class ListPartitionReassignmentsRequest_v0(Request):
    FLEXIBLE_VERSION: bool
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ListPartitionReassignmentsResponse_v0
    SCHEMA: Incomplete

ListPartitionReassignmentsRequest: Incomplete
ListPartitionReassignmentsResponse: Incomplete

class ElectLeadersResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ElectLeadersRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ElectLeadersResponse_v0
    SCHEMA: Incomplete

class ElectLeadersResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ElectLeadersRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ElectLeadersResponse_v1
    SCHEMA: Incomplete

class ElectionType(IntEnum):
    PREFERRED = 0
    UNCLEAN = 1

ElectLeadersRequest: Incomplete
ElectLeadersResponse: Incomplete



import abc
from _typeshed import Incomplete
from enum import IntEnum

log: Incomplete
NO_PRODUCER_ID: int
NO_PRODUCER_EPOCH: int
NO_SEQUENCE: int

class ProducerIdAndEpoch:
    producer_id: Incomplete
    epoch: Incomplete
    def __init__(self, producer_id, epoch) -> None: ...
    @property
    def is_valid(self): ...
    def match(self, batch): ...
    def __eq__(self, other): ...

class TransactionState(IntEnum):
    UNINITIALIZED = 0
    INITIALIZING = 1
    READY = 2
    IN_TRANSACTION = 3
    COMMITTING_TRANSACTION = 4
    ABORTING_TRANSACTION = 5
    ABORTABLE_ERROR = 6
    FATAL_ERROR = 7
    @classmethod
    def is_transition_valid(cls, source, target): ...

class Priority(IntEnum):
    FIND_COORDINATOR = 0
    INIT_PRODUCER_ID = 1
    ADD_PARTITIONS_OR_OFFSETS = 2
    END_TXN = 3

class TransactionManager:
    NO_INFLIGHT_REQUEST_CORRELATION_ID: int
    ADD_PARTITIONS_RETRY_BACKOFF_MS: int
    transactional_id: Incomplete
    transaction_timeout_ms: Incomplete
    producer_id_and_epoch: Incomplete
    retry_backoff_ms: Incomplete
    def __init__(
        self,
        transactional_id=None,
        transaction_timeout_ms: int = 0,
        retry_backoff_ms: int = 100,
        api_version=(0, 11),
        metadata=None,
    ) -> None: ...
    def initialize_transactions(self): ...
    def begin_transaction(self) -> None: ...
    def begin_commit(self): ...
    def begin_abort(self): ...
    def send_offsets_to_transaction(self, offsets, consumer_group_id): ...
    def maybe_add_partition_to_transaction(self, topic_partition) -> None: ...
    def is_send_to_partition_allowed(self, tp): ...
    def has_producer_id(self, producer_id=None): ...
    def is_transactional(self): ...
    def has_partitions_to_add(self): ...
    def is_completing(self): ...
    @property
    def last_error(self): ...
    def has_error(self): ...
    def is_aborting(self): ...
    def transition_to_abortable_error(self, exc) -> None: ...
    def transition_to_fatal_error(self, exc) -> None: ...
    def is_partition_added(self, partition): ...
    def is_partition_pending_add(self, partition): ...
    def has_producer_id_and_epoch(self, producer_id, producer_epoch): ...
    def set_producer_id_and_epoch(self, producer_id_and_epoch) -> None: ...
    def reset_producer_id(self) -> None: ...
    def sequence_number(self, tp): ...
    def increment_sequence_number(self, tp, increment) -> None: ...
    def reset_sequence_for_partition(self, tp) -> None: ...
    def next_request_handler(self, has_incomplete_batches): ...
    def retry(self, request) -> None: ...
    def authentication_failed(self, exc) -> None: ...
    def coordinator(self, coord_type): ...
    def lookup_coordinator_for_request(self, request) -> None: ...
    def next_in_flight_request_correlation_id(self): ...
    def clear_in_flight_transactional_request_correlation_id(self) -> None: ...
    def has_in_flight_transactional_request(self): ...
    def has_fatal_error(self): ...
    def has_abortable_error(self): ...

class TransactionalRequestResult:
    def __init__(self) -> None: ...
    def done(self, error=None) -> None: ...
    def wait(self, timeout_ms=None): ...
    @property
    def is_done(self): ...
    @property
    def succeeded(self): ...
    @property
    def failed(self): ...
    @property
    def exception(self): ...

class TxnRequestHandler(metaclass=abc.ABCMeta):
    transaction_manager: Incomplete
    retry_backoff_ms: Incomplete
    request: Incomplete
    def __init__(self, transaction_manager, result=None) -> None: ...
    @property
    def transactional_id(self): ...
    @property
    def producer_id(self): ...
    @property
    def producer_epoch(self): ...
    def fatal_error(self, exc) -> None: ...
    def abortable_error(self, exc) -> None: ...
    def fail(self, exc) -> None: ...
    def reenqueue(self) -> None: ...
    def on_complete(self, correlation_id, response_or_exc) -> None: ...
    def needs_coordinator(self): ...
    @property
    def result(self): ...
    @property
    def coordinator_type(self): ...
    @property
    def coordinator_key(self): ...
    def set_retry(self) -> None: ...
    @property
    def is_retry(self): ...
    @abc.abstractmethod
    def handle_response(self, response): ...
    @property
    @abc.abstractmethod
    def priority(self): ...

class InitProducerIdHandler(TxnRequestHandler):
    request: Incomplete
    def __init__(self, transaction_manager, transaction_timeout_ms) -> None: ...
    @property
    def priority(self): ...
    def handle_response(self, response) -> None: ...

class AddPartitionsToTxnHandler(TxnRequestHandler):
    request: Incomplete
    def __init__(self, transaction_manager, topic_partitions) -> None: ...
    @property
    def priority(self): ...
    retry_backoff_ms: Incomplete
    def handle_response(self, response) -> None: ...
    def maybe_override_retry_backoff_ms(self) -> None: ...

class FindCoordinatorHandler(TxnRequestHandler):
    request: Incomplete
    def __init__(self, transaction_manager, coord_type, coord_key) -> None: ...
    @property
    def priority(self): ...
    @property
    def coordinator_type(self) -> None: ...
    @property
    def coordinator_key(self) -> None: ...
    def handle_response(self, response) -> None: ...

class EndTxnHandler(TxnRequestHandler):
    request: Incomplete
    def __init__(self, transaction_manager, committed) -> None: ...
    @property
    def priority(self): ...
    def handle_response(self, response) -> None: ...

class AddOffsetsToTxnHandler(TxnRequestHandler):
    consumer_group_id: Incomplete
    offsets: Incomplete
    request: Incomplete
    def __init__(self, transaction_manager, consumer_group_id, offsets) -> None: ...
    @property
    def priority(self): ...
    def handle_response(self, response) -> None: ...

class TxnOffsetCommitHandler(TxnRequestHandler):
    consumer_group_id: Incomplete
    offsets: Incomplete
    request: Incomplete
    def __init__(self, transaction_manager, consumer_group_id, offsets, result) -> None: ...
    @property
    def priority(self): ...
    @property
    def coordinator_type(self): ...
    @property
    def coordinator_key(self): ...
    def handle_response(self, response) -> None: ...



from .client_mixin import OAuth2ClientMixin as OAuth2ClientMixin
from .functions import (
    create_bearer_token_validator as create_bearer_token_validator,
    create_query_client_func as create_query_client_func,
    create_query_token_func as create_query_token_func,
    create_revocation_endpoint as create_revocation_endpoint,
    create_save_token_func as create_save_token_func,
)
from .tokens_mixins import OAuth2AuthorizationCodeMixin as OAuth2AuthorizationCodeMixin, OAuth2TokenMixin as OAuth2TokenMixin

__all__ = [
    "OAuth2ClientMixin",
    "OAuth2AuthorizationCodeMixin",
    "OAuth2TokenMixin",
    "create_query_client_func",
    "create_save_token_func",
    "create_query_token_func",
    "create_revocation_endpoint",
    "create_bearer_token_validator",
]



from _typeshed import Incomplete, SupportsWrite
from collections.abc import Iterable
from typing_extensions import Never

from pygments.formatter import Formatter
from pygments.token import _TokenType

__all__ = ["ImageFormatter", "GifImageFormatter", "JpgImageFormatter", "BmpImageFormatter"]

class PilNotAvailable(ImportError): ...
class FontNotFound(Exception): ...

class FontManager:
    font_name: Incomplete
    font_size: Incomplete
    fonts: Incomplete
    encoding: Incomplete
    variable: bool
    def __init__(self, font_name, font_size: int = 14) -> None: ...
    def get_char_size(self): ...
    def get_text_size(self, text): ...
    def get_font(self, bold, oblique): ...
    def get_style(self, style): ...

class ImageFormatter(Formatter[bytes]):
    default_image_format: str
    encoding: str
    styles: Incomplete
    background_color: str
    image_format: Incomplete
    image_pad: Incomplete
    line_pad: Incomplete
    fonts: Incomplete
    line_number_fg: Incomplete
    line_number_bg: Incomplete
    line_number_chars: Incomplete
    line_number_bold: Incomplete
    line_number_italic: Incomplete
    line_number_pad: Incomplete
    line_numbers: Incomplete
    line_number_separator: Incomplete
    line_number_step: Incomplete
    line_number_start: Incomplete
    line_number_width: Incomplete
    hl_lines: Incomplete
    hl_color: Incomplete
    drawables: Incomplete
    def get_style_defs(self, arg: str = "") -> Never: ...
    def format(self, tokensource: Iterable[tuple[_TokenType, str]], outfile: SupportsWrite[bytes]) -> None: ...

class GifImageFormatter(ImageFormatter):
    default_image_format: str

class JpgImageFormatter(ImageFormatter):
    default_image_format: str

class BmpImageFormatter(ImageFormatter):
    default_image_format: str



from _typeshed import Incomplete
from typing import TypedDict, type_check_only

@type_check_only
class _DirtyErrorDict(TypedDict):
    error_type: str
    message: str
    details: dict[str, Incomplete]

class DirtyError(Exception):
    message: str
    details: dict[str, Incomplete]

    def __init__(self, message: str, details: dict[str, Incomplete] | None = None) -> None: ...
    def to_dict(self) -> _DirtyErrorDict: ...
    @classmethod
    def from_dict(cls, data: dict[str, Incomplete]) -> DirtyError: ...

class DirtyTimeoutError(DirtyError):
    timeout: float | None

    def __init__(self, message: str = "Operation timed out", timeout: float | None = None) -> None: ...

class DirtyConnectionError(DirtyError):
    socket_path: str | None

    def __init__(self, message: str = "Connection failed", socket_path: str | None = None) -> None: ...

class DirtyWorkerError(DirtyError):
    worker_id: int | None
    traceback: str | None

    def __init__(self, message: str, worker_id: int | None = None, traceback: str | None = None) -> None: ...

class DirtyAppError(DirtyError):
    app_path: str | None
    action: str | None
    traceback: str | None

    def __init__(
        self, message: str, app_path: str | None = None, action: str | None = None, traceback: str | None = None
    ) -> None: ...

class DirtyAppNotFoundError(DirtyAppError):
    app_path: str

    def __init__(self, app_path: str) -> None: ...

class DirtyNoWorkersAvailableError(DirtyError):
    app_path: str

    def __init__(self, app_path: str, message: str | None = None) -> None: ...

class DirtyProtocolError(DirtyError):
    def __init__(self, message: str = "Protocol error", raw_data: str | bytes | None = None) -> None: ...



import abc

from kafka.metrics.stat import AbstractStat

class AbstractCompoundStat(AbstractStat, metaclass=abc.ABCMeta):
    def stats(self) -> None: ...

class NamedMeasurable:
    def __init__(self, metric_name, measurable_stat) -> None: ...
    @property
    def name(self): ...
    @property
    def stat(self): ...



import abc
import asyncio
from _typeshed import Incomplete, MaybeNone
from collections.abc import (
    AsyncIterable,
    AsyncIterator,
    Awaitable,
    Callable,
    Collection,
    Generator,
    Iterable,
    Iterator,
    Mapping,
    Sequence,
)
from concurrent import futures
from types import TracebackType
from typing import Any, Generic, NoReturn, TypeAlias, TypeVar, overload, type_check_only
from typing_extensions import Self

from grpc import (
    CallCredentials,
    ChannelConnectivity,
    ChannelCredentials,
    Compression,
    GenericRpcHandler,
    HandlerCallDetails,
    RpcError,
    RpcMethodHandler,
    ServerCredentials,
    StatusCode,
    _Options,
)

_TRequest = TypeVar("_TRequest")
_TResponse = TypeVar("_TResponse")

# Exceptions:

class BaseError(Exception): ...
class UsageError(BaseError): ...
class AbortError(BaseError): ...
class InternalError(BaseError): ...

class AioRpcError(RpcError):
    def __init__(
        self,
        code: StatusCode,
        initial_metadata: Metadata,
        trailing_metadata: Metadata,
        details: str | None = None,
        debug_error_string: str | None = None,
    ) -> None: ...
    def debug_error_string(self) -> str: ...
    def initial_metadata(self) -> Metadata | MaybeNone: ...
    # AioRpcError returns the async Metadata, overriding the synchronous
    # grpc.RpcError.trailing_metadata() -> tuple[_Metadatum, ...].
    def trailing_metadata(self) -> Metadata | MaybeNone: ...  # type: ignore[override]

# Create Client:

def insecure_channel(
    target: str,
    options: _Options | None = None,
    compression: Compression | None = None,
    interceptors: Sequence[ClientInterceptor] | None = None,
) -> Channel: ...
def secure_channel(
    target: str,
    credentials: ChannelCredentials,
    options: _Options | None = None,
    compression: Compression | None = None,
    interceptors: Sequence[ClientInterceptor] | None = None,
) -> Channel: ...

# Create Server:

def server(
    migration_thread_pool: futures.Executor | None = None,
    handlers: Sequence[GenericRpcHandler] | None = None,
    interceptors: Sequence[ServerInterceptor] | None = None,
    options: _Options | None = None,
    maximum_concurrent_rpcs: int | None = None,
    compression: Compression | None = None,
) -> Server: ...

# Channel Object:

_Serializer: TypeAlias = Callable[[_T], bytes]
_Deserializer: TypeAlias = Callable[[bytes], _T]

class Channel(abc.ABC):
    @abc.abstractmethod
    async def close(self, grace: float | None = None) -> None: ...
    @abc.abstractmethod
    def get_state(self, try_to_connect: bool = False) -> ChannelConnectivity: ...
    @abc.abstractmethod
    async def wait_for_state_change(self, last_observed_state: ChannelConnectivity) -> None: ...
    @abc.abstractmethod
    def stream_stream(
        self,
        method: str,
        request_serializer: _Serializer[_TRequest] | None = None,
        response_deserializer: _Deserializer[_TResponse] | None = None,
    ) -> StreamStreamMultiCallable[_TRequest, _TResponse]: ...
    @abc.abstractmethod
    def stream_unary(
        self,
        method: str,
        request_serializer: _Serializer[_TRequest] | None = None,
        response_deserializer: _Deserializer[_TResponse] | None = None,
    ) -> StreamUnaryMultiCallable[_TRequest, _TResponse]: ...
    @abc.abstractmethod
    def unary_stream(
        self,
        method: str,
        request_serializer: _Serializer[_TRequest] | None = None,
        response_deserializer: _Deserializer[_TResponse] | None = None,
    ) -> UnaryStreamMultiCallable[_TRequest, _TResponse]: ...
    @abc.abstractmethod
    def unary_unary(
        self,
        method: str,
        request_serializer: _Serializer[_TRequest] | None = None,
        response_deserializer: _Deserializer[_TResponse] | None = None,
    ) -> UnaryUnaryMultiCallable[_TRequest, _TResponse]: ...
    @abc.abstractmethod
    async def __aenter__(self) -> Self: ...
    @abc.abstractmethod
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> bool | None: ...
    @abc.abstractmethod
    async def channel_ready(self) -> None: ...

# Server Object:

class Server(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def add_generic_rpc_handlers(self, generic_rpc_handlers: Iterable[GenericRpcHandler]) -> None: ...

    # Returns an integer port on which server will accept RPC requests.
    @abc.abstractmethod
    def add_insecure_port(self, address: str) -> int: ...

    # Returns an integer port on which server will accept RPC requests.
    @abc.abstractmethod
    def add_secure_port(self, address: str, server_credentials: ServerCredentials) -> int: ...
    @abc.abstractmethod
    async def start(self) -> None: ...

    # Grace period is in seconds.
    @abc.abstractmethod
    async def stop(self, grace: float | None) -> None: ...

    # Returns a bool indicates if the operation times out. Timeout is in seconds.
    @abc.abstractmethod
    async def wait_for_termination(self, timeout: float | None = None) -> bool: ...

# Client-Side Context:

_DoneCallbackType: TypeAlias = Callable[[Any], None]
_EOFType: TypeAlias = object

class RpcContext(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def cancelled(self) -> bool: ...
    @abc.abstractmethod
    def done(self) -> bool: ...
    @abc.abstractmethod
    def time_remaining(self) -> float | None: ...
    @abc.abstractmethod
    def cancel(self) -> bool: ...
    @abc.abstractmethod
    def add_done_callback(self, callback: _DoneCallbackType) -> None: ...

class Call(RpcContext, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    async def initial_metadata(self) -> Metadata: ...
    @abc.abstractmethod
    async def trailing_metadata(self) -> Metadata: ...
    @abc.abstractmethod
    async def code(self) -> StatusCode: ...
    @abc.abstractmethod
    async def details(self) -> str: ...
    @abc.abstractmethod
    async def wait_for_connection(self) -> None: ...

class UnaryUnaryCall(Call, Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __await__(self) -> Generator[None, None, _TResponse]: ...

class UnaryStreamCall(Call, Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __aiter__(self) -> AsyncIterator[_TResponse]: ...
    @abc.abstractmethod
    async def read(self) -> _EOFType | _TResponse: ...

class StreamUnaryCall(Call, Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    async def write(self, request: _TRequest) -> None: ...
    @abc.abstractmethod
    async def done_writing(self) -> None: ...
    @abc.abstractmethod
    def __await__(self) -> Generator[None, None, _TResponse]: ...

class StreamStreamCall(Call, Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __aiter__(self) -> AsyncIterator[_TResponse]: ...
    @abc.abstractmethod
    async def read(self) -> _EOFType | _TResponse: ...
    @abc.abstractmethod
    async def write(self, request: _TRequest) -> None: ...
    @abc.abstractmethod
    async def done_writing(self) -> None: ...

# Service-Side Context:

@type_check_only
class _DoneCallback(Generic[_TRequest, _TResponse]):
    def __call__(self, ctx: ServicerContext[_TRequest, _TResponse]) -> None: ...

class ServicerContext(Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    async def abort(self, code: StatusCode, details: str = "", trailing_metadata: _MetadataType = ()) -> NoReturn: ...
    @abc.abstractmethod
    async def read(self) -> _TRequest: ...
    @abc.abstractmethod
    async def write(self, message: _TResponse) -> None: ...
    @abc.abstractmethod
    async def send_initial_metadata(self, initial_metadata: _MetadataType) -> None: ...
    def add_done_callback(self, callback: _DoneCallback[_TRequest, _TResponse]) -> None: ...
    @abc.abstractmethod
    def set_trailing_metadata(self, trailing_metadata: _MetadataType) -> None: ...
    @abc.abstractmethod
    def invocation_metadata(self) -> Metadata | None: ...
    @abc.abstractmethod
    def set_code(self, code: StatusCode) -> None: ...
    @abc.abstractmethod
    def set_details(self, details: str) -> None: ...
    @abc.abstractmethod
    def set_compression(self, compression: Compression) -> None: ...
    @abc.abstractmethod
    def disable_next_message_compression(self) -> None: ...
    @abc.abstractmethod
    def peer(self) -> str: ...
    @abc.abstractmethod
    def peer_identities(self) -> Iterable[bytes] | None: ...
    @abc.abstractmethod
    def peer_identity_key(self) -> str | None: ...
    @abc.abstractmethod
    def auth_context(self) -> Mapping[str, Iterable[bytes]]: ...
    def time_remaining(self) -> float: ...
    def trailing_metadata(self) -> Metadata: ...
    def code(self) -> StatusCode: ...
    def details(self) -> str: ...
    def cancelled(self) -> bool: ...
    def done(self) -> bool: ...

# Client-Side Interceptor:

class ClientCallDetails(abc.ABC):
    def __new__(
        _cls,
        method: str,
        timeout: float | None,
        metadata: Metadata | None,
        credentials: CallCredentials | None,
        wait_for_ready: bool | None,
    ) -> Self: ...

    method: str
    timeout: float | None
    metadata: Metadata | None
    credentials: CallCredentials | None

    # "This is an EXPERIMENTAL argument. An optional flag t enable wait for ready mechanism."
    wait_for_ready: bool | None

    # As at 1.53.0, this is not supported in aio:
    # compression: Compression | None

@type_check_only
class _InterceptedCall(Generic[_TRequest, _TResponse]):
    def __init__(self, interceptors_task: asyncio.Task[Any]) -> None: ...
    def __del__(self) -> None: ...
    def cancel(self) -> bool: ...
    def cancelled(self) -> bool: ...
    def done(self) -> bool: ...
    def add_done_callback(self, callback: _DoneCallback[_TRequest, _TResponse]) -> None: ...
    def time_remaining(self) -> float | None: ...
    async def initial_metadata(self) -> Metadata | None: ...
    async def trailing_metadata(self) -> Metadata | None: ...
    async def code(self) -> StatusCode: ...
    async def details(self) -> str: ...
    async def debug_error_string(self) -> str | None: ...
    async def wait_for_connection(self) -> None: ...

class InterceptedUnaryUnaryCall(_InterceptedCall[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    def __await__(self) -> Generator[Incomplete, None, _TResponse]: ...
    def __init__(
        self,
        interceptors: Sequence[UnaryUnaryClientInterceptor],
        request: _TRequest,
        timeout: float | None,
        metadata: Metadata,
        credentials: CallCredentials | None,
        wait_for_ready: bool | None,
        channel: Channel,
        method: bytes,
        request_serializer: _Serializer[_TRequest],
        response_deserializer: _Deserializer[_TResponse],
        loop: asyncio.AbstractEventLoop,
    ) -> None: ...

    # pylint: disable=too-many-arguments
    async def _invoke(
        self,
        interceptors: Sequence[UnaryUnaryClientInterceptor],
        method: bytes,
        timeout: float | None,
        metadata: Metadata | None,
        credentials: CallCredentials | None,
        wait_for_ready: bool | None,
        request: _TRequest,
        request_serializer: _Serializer[_TRequest],
        response_deserializer: _Deserializer[_TResponse],
    ) -> UnaryUnaryCall[_TRequest, _TResponse]: ...
    def time_remaining(self) -> float | None: ...

class ClientInterceptor(metaclass=abc.ABCMeta): ...

class UnaryUnaryClientInterceptor(ClientInterceptor, metaclass=abc.ABCMeta):
    # This method (not the class) is generic over _TRequest and _TResponse
    # and the types must satisfy the no-op implementation of
    # `return await continuation(client_call_details, request)`.
    @abc.abstractmethod
    async def intercept_unary_unary(
        self,
        continuation: Callable[[ClientCallDetails, _TRequest], Awaitable[UnaryUnaryCall[_TRequest, _TResponse]]],
        client_call_details: ClientCallDetails,
        request: _TRequest,
    ) -> _TResponse | UnaryUnaryCall[_TRequest, _TResponse]: ...

class UnaryStreamClientInterceptor(ClientInterceptor, metaclass=abc.ABCMeta):
    # This method (not the class) is generic over _TRequest and _TResponse
    # and the types must satisfy the no-op implementation of
    # `return await continuation(client_call_details, request)`.
    @abc.abstractmethod
    async def intercept_unary_stream(
        self,
        continuation: Callable[[ClientCallDetails, _TRequest], Awaitable[UnaryStreamCall[_TRequest, _TResponse]]],
        client_call_details: ClientCallDetails,
        request: _TRequest,
    ) -> AsyncIterator[_TResponse] | UnaryStreamCall[_TRequest, _TResponse]: ...

class StreamUnaryClientInterceptor(ClientInterceptor, metaclass=abc.ABCMeta):
    # This method (not the class) is generic over _TRequest and _TResponse
    # and the types must satisfy the no-op implementation of
    # `return await continuation(client_call_details, request_iterator)`.
    @abc.abstractmethod
    async def intercept_stream_unary(
        self,
        continuation: Callable[
            [ClientCallDetails, AsyncIterable[_TRequest] | Iterable[_TRequest]], Awaitable[StreamUnaryCall[_TRequest, _TResponse]]
        ],
        client_call_details: ClientCallDetails,
        request_iterator: AsyncIterable[_TRequest] | Iterable[_TRequest],
    ) -> _TResponse | StreamUnaryCall[_TRequest, _TResponse]: ...

class StreamStreamClientInterceptor(ClientInterceptor, metaclass=abc.ABCMeta):
    # This method (not the class) is generic over _TRequest and _TResponse
    # and the types must satisfy the no-op implementation of
    # `return await continuation(client_call_details, request_iterator)`.
    @abc.abstractmethod
    async def intercept_stream_stream(
        self,
        continuation: Callable[
            [ClientCallDetails, AsyncIterable[_TRequest] | Iterable[_TRequest]],
            Awaitable[StreamStreamCall[_TRequest, _TResponse]],
        ],
        client_call_details: ClientCallDetails,
        request_iterator: AsyncIterable[_TRequest] | Iterable[_TRequest],
    ) -> AsyncIterator[_TResponse] | StreamStreamCall[_TRequest, _TResponse]: ...

# Server-Side Interceptor:

class ServerInterceptor(metaclass=abc.ABCMeta):
    # This method (not the class) is generic over _TRequest and _TResponse
    # and the types must satisfy the no-op implementation of
    # `return await continuation(handler_call_details)`.
    # The return is Optional: per the runtime docstring, an interceptor
    # may return None to signal that the RPC is not serviced, and the
    # continuation propagates that None down the chain.
    @abc.abstractmethod
    async def intercept_service(
        self,
        continuation: Callable[[HandlerCallDetails], Awaitable[RpcMethodHandler[_TRequest, _TResponse] | None]],
        handler_call_details: HandlerCallDetails,
    ) -> RpcMethodHandler[_TRequest, _TResponse] | None: ...

# Multi-Callable Interfaces:

class UnaryUnaryMultiCallable(Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __call__(
        self,
        request: _TRequest,
        *,
        timeout: float | None = None,
        metadata: _MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> UnaryUnaryCall[_TRequest, _TResponse]: ...

class UnaryStreamMultiCallable(Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __call__(
        self,
        request: _TRequest,
        *,
        timeout: float | None = None,
        metadata: _MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> UnaryStreamCall[_TRequest, _TResponse]: ...

class StreamUnaryMultiCallable(Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __call__(
        self,
        request_iterator: AsyncIterator[_TRequest] | Iterator[_TRequest] | None = None,
        timeout: float | None = None,
        metadata: _MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> StreamUnaryCall[_TRequest, _TResponse]: ...

class StreamStreamMultiCallable(Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __call__(
        self,
        request_iterator: AsyncIterator[_TRequest] | Iterator[_TRequest] | None = None,
        timeout: float | None = None,
        metadata: _MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> StreamStreamCall[_TRequest, _TResponse]: ...

# Metadata:

_MetadataKey: TypeAlias = str
_MetadataValue: TypeAlias = str | bytes
_MetadatumType: TypeAlias = tuple[_MetadataKey, _MetadataValue]
_MetadataType: TypeAlias = Metadata | Sequence[_MetadatumType]
_T = TypeVar("_T")

class Metadata(Collection[_MetadatumType]):
    def __init__(self, *args: tuple[_MetadataKey, _MetadataValue]) -> None: ...
    @classmethod
    def from_tuple(cls, raw_metadata: tuple[_MetadataKey, _MetadataValue]) -> Metadata: ...
    def add(self, key: _MetadataKey, value: _MetadataValue) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: _MetadataKey) -> _MetadataValue: ...
    def __setitem__(self, key: _MetadataKey, value: _MetadataValue) -> None: ...
    def __delitem__(self, key: _MetadataKey) -> None: ...
    def delete_all(self, key: _MetadataKey) -> None: ...
    def __iter__(self) -> Iterator[_MetadatumType]: ...

    @overload
    def get(self, key: _MetadataKey, default: None = None) -> _MetadataValue | None: ...
    @overload
    def get(self, key: _MetadataKey, default: _MetadataValue) -> _MetadataValue: ...
    @overload
    def get(self, key: _MetadataKey, default: _T) -> _MetadataValue | _T: ...

    def get_all(self, key: _MetadataKey) -> list[_MetadataValue]: ...
    def set_all(self, key: _MetadataKey, values: list[_MetadataValue]) -> None: ...
    def __contains__(self, key: object) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __add__(self, other: Any) -> Metadata: ...



from kafka.metrics.stats.sampled_stat import AbstractSampledStat

class Count(AbstractSampledStat):
    def __init__(self) -> None: ...
    def update(self, sample, config, value, now) -> None: ...
    def combine(self, samples, config, now): ...



from . import stash as stash
from .app import DirtyApp as DirtyApp
from .arbiter import DirtyArbiter as DirtyArbiter
from .client import (
    DirtyClient as DirtyClient,
    close_dirty_client as close_dirty_client,
    close_dirty_client_async as close_dirty_client_async,
    get_dirty_client as get_dirty_client,
    get_dirty_client_async as get_dirty_client_async,
    set_dirty_socket_path as set_dirty_socket_path,
)
from .errors import (
    DirtyAppError as DirtyAppError,
    DirtyAppNotFoundError as DirtyAppNotFoundError,
    DirtyConnectionError as DirtyConnectionError,
    DirtyError as DirtyError,
    DirtyProtocolError as DirtyProtocolError,
    DirtyTimeoutError as DirtyTimeoutError,
    DirtyWorkerError as DirtyWorkerError,
)
from .stash import (
    StashClient as StashClient,
    StashError as StashError,
    StashKeyNotFoundError as StashKeyNotFoundError,
    StashTable as StashTable,
    StashTableNotFoundError as StashTableNotFoundError,
)

__all__ = [
    "DirtyError",
    "DirtyTimeoutError",
    "DirtyConnectionError",
    "DirtyWorkerError",
    "DirtyAppError",
    "DirtyAppNotFoundError",
    "DirtyProtocolError",
    "DirtyApp",
    "DirtyClient",
    "get_dirty_client",
    "get_dirty_client_async",
    "close_dirty_client",
    "close_dirty_client_async",
    "stash",
    "StashClient",
    "StashTable",
    "StashError",
    "StashTableNotFoundError",
    "StashKeyNotFoundError",
    "DirtyArbiter",
    "set_dirty_socket_path",
]



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["center", "centroid"]

def center(G: Graph[_Node]) -> list[Incomplete]: ...
@_dispatchable
def centroid(G: Graph[_Node]) -> list[Incomplete]: ...



from _typeshed import Incomplete

from kafka.protocol.api import Request, Response

class SaslAuthenticateResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class SaslAuthenticateResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class SaslAuthenticateRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = SaslAuthenticateResponse_v0
    SCHEMA: Incomplete

class SaslAuthenticateRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = SaslAuthenticateResponse_v1
    SCHEMA: Incomplete

SaslAuthenticateRequest: Incomplete
SaslAuthenticateResponse: Incomplete



from _typeshed import Incomplete

from .spreadsheet_drawing import AbsoluteAnchor, OneCellAnchor

class Drawing:
    count: int
    name: str
    description: str
    coordinates: Incomplete
    left: int
    top: int
    resize_proportional: bool
    rotation: int
    anchortype: str
    anchorcol: int
    anchorrow: int
    def __init__(self) -> None: ...

    @property
    def width(self) -> int: ...
    @width.setter
    def width(self, w: int) -> None: ...

    @property
    def height(self) -> int: ...
    @height.setter
    def height(self, h: int) -> None: ...

    def set_dimension(self, w: int = 0, h: int = 0) -> None: ...
    @property
    def anchor(self) -> AbsoluteAnchor | OneCellAnchor: ...



from _typeshed import Incomplete, ReadableBuffer
from collections.abc import Iterable
from typing import ClassVar, Literal, SupportsBytes, SupportsIndex
from typing_extensions import Self

from authlib.jose.rfc7517 import Key

class AsymmetricKey(Key):
    PUBLIC_KEY_FIELDS: ClassVar[list[str]]
    PRIVATE_KEY_FIELDS: ClassVar[list[str]]
    PRIVATE_KEY_CLS: ClassVar[type | tuple[type, ...]]
    PUBLIC_KEY_CLS: ClassVar[type | tuple[type, ...]]
    SSH_PUBLIC_PREFIX: ClassVar[bytes]
    private_key: Incomplete
    public_key: Incomplete
    def __init__(self, private_key=None, public_key=None, options=None) -> None: ...
    @property
    def public_only(self) -> bool: ...
    def get_op_key(self, operation): ...
    def get_public_key(self): ...
    def get_private_key(self): ...
    def load_raw_key(self) -> None: ...
    def load_dict_key(self) -> None: ...
    def dumps_private_key(self): ...
    def dumps_public_key(self): ...
    def load_private_key(self): ...
    def load_public_key(self): ...
    def as_dict(self, is_private: bool = False, **params) -> dict[Incomplete, Incomplete]: ...
    def as_key(self, is_private: bool = False): ...
    def as_bytes(
        self,
        encoding: Literal["PEM", "DER"] | None = None,
        is_private: bool = False,
        password: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer | None = None,
    ): ...
    def as_pem(
        self,
        is_private: bool = False,
        password: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer | None = None,
    ): ...
    def as_der(
        self,
        is_private: bool = False,
        password: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer | None = None,
    ): ...
    @classmethod
    def import_dict_key(cls, raw, options=None) -> Self: ...
    @classmethod
    def import_key(cls, raw, options=None) -> Self: ...
    @classmethod
    def validate_raw_key(cls, key) -> bool: ...
    @classmethod
    def generate_key(cls, crv_or_size, options=None, is_private: bool = False) -> AsymmetricKey: ...



from typing import Final

from .async_connection import AsyncHTTP2Connection
from .connection import HTTP2ServerConnection

H2_MIN_VERSION: Final[tuple[int, int, int]]

def is_http2_available() -> bool: ...
def get_h2_version() -> tuple[int, int, int]: ...
def get_http2_connection_class() -> type[HTTP2ServerConnection]: ...
def get_async_http2_connection_class() -> type[AsyncHTTP2Connection]: ...

__all__ = [
    "is_http2_available",
    "get_h2_version",
    "get_http2_connection_class",
    "get_async_http2_connection_class",
    "H2_MIN_VERSION",
]



class KafkaBytes(bytearray):
    def __init__(self, size) -> None: ...
    def read(self, nbytes=None): ...
    def write(self, data) -> None: ...
    def seek(self, idx) -> None: ...
    def tell(self): ...



from collections.abc import Callable
from typing import TypeAlias, TypeVar

from google.protobuf.descriptor_pool import DescriptorPool
from google.protobuf.message import Message

_MessageT = TypeVar("_MessageT", bound=Message)
_MsgFormatter: TypeAlias = Callable[[Message, int, bool], str | None]

def serialize(
    message: Message,
    as_utf8: bool = True,
    as_one_line: bool = False,
    use_short_repeated_primitives: bool = False,
    pointy_brackets: bool = False,
    use_index_order: bool = False,
    use_field_number: bool = False,
    descriptor_pool: DescriptorPool | None = None,
    indent: int = 0,
    message_formatter: _MsgFormatter | None = None,
    print_unknown_fields: bool = False,
    force_colon: bool = False,
) -> str: ...
def parse(
    message_class: type[_MessageT],
    text: str | bytes,
    allow_unknown_extension: bool = False,
    allow_field_number: bool = False,
    descriptor_pool: DescriptorPool | None = None,
    allow_unknown_field: bool = False,
) -> _MessageT: ...



from _typeshed import Incomplete

from kafka.record.abc import ABCRecords

class MemoryRecords(ABCRecords):
    LENGTH_OFFSET: Incomplete
    LOG_OVERHEAD: Incomplete
    MAGIC_OFFSET: Incomplete
    MIN_SLICE: Incomplete
    def __init__(self, bytes_data) -> None: ...
    def size_in_bytes(self): ...
    def valid_bytes(self): ...
    def has_next(self): ...
    def next_batch(self, _min_slice=26, _magic_offset=16): ...
    def __iter__(self): ...
    def __next__(self): ...
    next = __next__

class MemoryRecordsBuilder:
    def __init__(
        self,
        magic,
        compression_type,
        batch_size,
        offset: int = 0,
        transactional: bool = False,
        producer_id: int = -1,
        producer_epoch: int = -1,
        base_sequence: int = -1,
    ) -> None: ...
    def skip(self, offsets_to_skip) -> None: ...
    def append(self, timestamp, key, value, headers=[]): ...
    def set_producer_state(self, producer_id, producer_epoch, base_sequence, is_transactional) -> None: ...
    @property
    def producer_id(self): ...
    @property
    def producer_epoch(self): ...
    def records(self): ...
    def close(self) -> None: ...
    def size_in_bytes(self): ...
    def compression_rate(self): ...
    def is_full(self): ...
    def next_offset(self): ...
    def buffer(self): ...



from typing import Final

class HTTP2ErrorCode:
    NO_ERROR: Final = 0x0
    PROTOCOL_ERROR: Final = 0x1
    INTERNAL_ERROR: Final = 0x2
    FLOW_CONTROL_ERROR: Final = 0x3
    SETTINGS_TIMEOUT: Final = 0x4
    STREAM_CLOSED: Final = 0x5
    FRAME_SIZE_ERROR: Final = 0x6
    REFUSED_STREAM: Final = 0x7
    CANCEL: Final = 0x8
    COMPRESSION_ERROR: Final = 0x9
    CONNECT_ERROR: Final = 0xA
    ENHANCE_YOUR_CALM: Final = 0xB
    INADEQUATE_SECURITY: Final = 0xC
    HTTP_1_1_REQUIRED: Final = 0xD

class HTTP2Error(Exception):
    message: str
    error_code: int

    def __init__(self, message: str | None = None, error_code: int | None = None) -> None: ...

class HTTP2ProtocolError(HTTP2Error): ...
class HTTP2InternalError(HTTP2Error): ...
class HTTP2FlowControlError(HTTP2Error): ...
class HTTP2SettingsTimeout(HTTP2Error): ...
class HTTP2StreamClosed(HTTP2Error): ...
class HTTP2FrameSizeError(HTTP2Error): ...
class HTTP2RefusedStream(HTTP2Error): ...
class HTTP2Cancel(HTTP2Error): ...
class HTTP2CompressionError(HTTP2Error): ...
class HTTP2ConnectError(HTTP2Error): ...
class HTTP2EnhanceYourCalm(HTTP2Error): ...
class HTTP2InadequateSecurity(HTTP2Error): ...
class HTTP2RequiresHTTP11(HTTP2Error): ...

class HTTP2StreamError(HTTP2Error):
    stream_id: int

    def __init__(self, stream_id: int, message: str | None = None, error_code: int | None = None) -> None: ...

class HTTP2ConnectionError(HTTP2Error): ...
class HTTP2ConfigurationError(HTTP2Error): ...

class HTTP2NotAvailable(HTTP2Error):
    def __init__(self, message: str | None = None) -> None: ...

__all__ = [
    "HTTP2ErrorCode",
    "HTTP2Error",
    "HTTP2ProtocolError",
    "HTTP2InternalError",
    "HTTP2FlowControlError",
    "HTTP2SettingsTimeout",
    "HTTP2StreamClosed",
    "HTTP2FrameSizeError",
    "HTTP2RefusedStream",
    "HTTP2Cancel",
    "HTTP2CompressionError",
    "HTTP2ConnectError",
    "HTTP2EnhanceYourCalm",
    "HTTP2InadequateSecurity",
    "HTTP2RequiresHTTP11",
    "HTTP2StreamError",
    "HTTP2ConnectionError",
    "HTTP2ConfigurationError",
    "HTTP2NotAvailable",
]



import sys
from _typeshed import Incomplete
from socket import AddressFamily, SocketKind
from typing import Any, NamedTuple

# All named tuples are defined in this file, but due to the inability to detect some platforms,
# it was decided to store the correct named tuples inside platform-specific files.

class sswap(NamedTuple):
    total: int
    used: int
    free: int
    percent: float
    sin: int
    sout: int

class sdiskusage(NamedTuple):
    total: int
    used: int
    free: int
    percent: float

# redefine for linux:
if sys.platform != "linux":
    class sdiskio(NamedTuple):
        read_count: int
        write_count: int
        read_bytes: int
        write_bytes: int
        read_time: int
        write_time: int

class sdiskpart(NamedTuple):
    device: str
    mountpoint: str
    fstype: str
    opts: str

class snetio(NamedTuple):
    bytes_sent: int
    bytes_recv: int
    packets_sent: int
    packets_recv: int
    errin: int
    errout: int
    dropin: int
    dropout: int

class suser(NamedTuple):
    name: str
    terminal: str | None
    host: str | None
    started: float
    pid: str

class sconn(NamedTuple):
    fd: int
    family: AddressFamily
    type: SocketKind
    laddr: addr | tuple[()]
    raddr: addr | tuple[()]
    status: str
    pid: int | None

class snicaddr(NamedTuple):
    family: AddressFamily
    address: str
    netmask: str | None
    broadcast: str | None
    ptp: str | None

class snicstats(NamedTuple):
    isup: bool
    duplex: int
    speed: int
    mtu: int
    flags: str

class scpustats(NamedTuple):
    ctx_switches: int
    interrupts: int
    soft_interrupts: int
    syscalls: int

class scpufreq(NamedTuple):
    current: float
    min: float
    max: float

class shwtemp(NamedTuple):
    label: str
    current: float
    high: float | None
    critical: float | None

class sbattery(NamedTuple):
    percent: int
    secsleft: int
    power_plugged: bool

class sfan(NamedTuple):
    label: str
    current: int

if sys.platform == "win32":
    class pheap(NamedTuple):
        heap_used: Incomplete
        mmap_used: Incomplete
        heap_count: Incomplete

else:
    # if LINUX or MACOS or BSD:
    class pheap(NamedTuple):
        heap_used: Incomplete
        mmap_used: Incomplete

# redefine for linux:
if sys.platform != "linux":
    class pcputimes(NamedTuple):
        user: float
        system: float
        children_user: float
        children_system: float

    class popenfile(NamedTuple):
        path: str
        fd: int

class pthread(NamedTuple):
    id: int
    user_time: float
    system_time: float

class puids(NamedTuple):
    real: int
    effective: int
    saved: int

class pgids(NamedTuple):
    real: int
    effective: int
    saved: int

# redefine for linux and windows:
if sys.platform != "linux" and sys.platform != "win32":
    class pio(NamedTuple):
        read_count: int
        write_count: int
        read_bytes: int
        write_bytes: int

class pionice(NamedTuple):
    ioclass: int
    value: int

class pctxsw(NamedTuple):
    voluntary: int
    involuntary: int

class pconn(NamedTuple):
    fd: int
    family: AddressFamily
    type: SocketKind
    laddr: addr
    raddr: addr
    status: str

class addr(NamedTuple):
    ip: str
    port: int

if sys.platform == "linux":
    class scputimes(NamedTuple):
        # Note: scputimes has different fields depending on exactly how Linux
        # is setup, but we'll include the "complete" set of fields
        user: float
        nice: float
        system: float
        idle: float
        iowait: float
        irq: float
        softirq: float
        steal: float
        guest: float
        guest_nice: float

    class svmem(NamedTuple):
        total: int
        available: int
        percent: float
        used: int
        free: int
        active: int
        inactive: int
        buffers: int
        cached: int
        shared: int
        slab: int

    class sdiskio(NamedTuple):
        read_count: int
        write_count: int
        read_bytes: int
        write_bytes: int
        read_time: int
        write_time: int
        read_merged_count: int
        write_merged_count: int
        busy_time: int

    class popenfile(NamedTuple):
        path: str
        fd: int
        position: int
        mode: str
        flags: int

    class pmem(NamedTuple):
        rss: int
        vms: int
        shared: int
        text: int
        lib: int
        data: int
        dirty: int

    class pfullmem(NamedTuple):
        rss: int
        vms: int
        shared: int
        text: int
        lib: int
        data: int
        dirty: int
        uss: int
        pss: int
        swap: int

    class pmmap_grouped(NamedTuple):
        path: Incomplete
        rss: Incomplete
        size: Incomplete
        pss: Incomplete
        shared_clean: Incomplete
        shared_dirty: Incomplete
        private_clean: Incomplete
        private_dirty: Incomplete
        referenced: Incomplete
        anonymous: Incomplete
        swap: Incomplete

    class pmmap_ext(NamedTuple):
        addr: Incomplete
        perms: Incomplete
        path: Incomplete
        rss: Incomplete
        size: Incomplete
        pss: Incomplete
        shared_clean: Incomplete
        shared_dirty: Incomplete
        private_clean: Incomplete
        private_dirty: Incomplete
        referenced: Incomplete
        anonymous: Incomplete
        swap: Incomplete

    class pio(NamedTuple):
        read_count: int
        write_count: int
        read_bytes: int
        write_bytes: int
        read_chars: int
        write_chars: int

    class pcputimes(NamedTuple):
        user: float
        system: float
        children_user: float
        children_system: float
        iowait: float

elif sys.platform == "win32":
    class scputimes(NamedTuple):
        user: float
        system: float
        idle: float
        interrupt: float
        dpc: float

    class svmem(NamedTuple):
        total: int
        available: int
        percent: float
        used: int
        free: int

    class pmem(NamedTuple):
        rss: int
        vms: int
        num_page_faults: int
        peak_wset: int
        wset: int
        peak_paged_pool: int
        paged_pool: int
        peak_nonpaged_pool: int
        nonpaged_pool: int
        pagefile: int
        peak_pagefile: int
        private: int

    class pfullmem(NamedTuple):
        rss: int
        vms: int
        num_page_faults: int
        peak_wset: int
        wset: int
        peak_paged_pool: int
        paged_pool: int
        peak_nonpaged_pool: int
        nonpaged_pool: int
        pagefile: int
        peak_pagefile: int
        private: int
        uss: int

    class pmmap_grouped(NamedTuple):
        path: Incomplete
        rss: Incomplete

    class pmmap_ext(NamedTuple):
        addr: Incomplete
        perms: Incomplete
        path: Incomplete
        rss: Incomplete

    class pio(NamedTuple):
        read_count: int
        write_count: int
        read_bytes: int
        write_bytes: int
        other_count: int
        other_bytes: int

elif sys.platform == "darwin":
    class scputimes(NamedTuple):
        user: float
        nice: float
        system: float
        idle: float

    class svmem(NamedTuple):
        total: int
        available: int
        percent: float
        used: int
        free: int
        active: int
        inactive: int
        wired: int

    class pmem(NamedTuple):
        rss: int
        vms: int
        pfaults: int
        pageins: int

    class pfullmem(NamedTuple):
        rss: int
        vms: int
        pfaults: int
        pageins: int
        uss: int

else:
    # See _psbsd.pyi, _pssunos.pyi or _psaix.pyi
    # BSD: svmem, scputimes, pmem, pfullmem, pcputimes, pmmap_grouped, pmmap_ext, sdiskio
    # SUNOS: scputimes, pcputimes, svmem, pmem, pfullmem, pmmap_grouped, pmmap_ext
    # AIX: pmem, pfullmem, scputimes, svmem

    scputimes = Incomplete

    class pmem(Any): ...
    class pfullmem(Any): ...
    class svmem(Any): ...



from _typeshed import StrOrBytesPath
from collections.abc import Sequence

from _remote_debugging import AwaitedInfo, InterpreterInfo

from .collector import Collector, _Frame, _Timestamps

class HeatmapCollector(Collector):
    FILE_INDEX_FORMAT: str
    def __init__(self, sample_interval_usec: int, *, skip_idle: bool = False) -> None: ...
    def collect(
        self, stack_frames: Sequence[InterpreterInfo] | Sequence[AwaitedInfo], timestamps_us: _Timestamps = None
    ) -> None: ...
    def export(self, output_path: StrOrBytesPath) -> None: ...
    def process_frames(self, frames: Sequence[_Frame], thread_id: int, weight: int = 1) -> None: ...
    def set_stats(
        self,
        sample_interval_usec: int,
        duration_sec: float,
        sample_rate: float,
        error_rate: float | None = None,
        missed_samples: float | None = None,
        **kwargs: object,
    ) -> None: ...



from google.protobuf.message import Message

TAG_TYPE_BITS: int
TAG_TYPE_MASK: int
WIRETYPE_VARINT: int
WIRETYPE_FIXED64: int
WIRETYPE_LENGTH_DELIMITED: int
WIRETYPE_START_GROUP: int
WIRETYPE_END_GROUP: int
WIRETYPE_FIXED32: int
INT32_MAX: int
INT32_MIN: int
UINT32_MAX: int
INT64_MAX: int
INT64_MIN: int
UINT64_MAX: int
FORMAT_UINT32_LITTLE_ENDIAN: str
FORMAT_UINT64_LITTLE_ENDIAN: str
FORMAT_FLOAT_LITTLE_ENDIAN: str
FORMAT_DOUBLE_LITTLE_ENDIAN: str

def PackTag(field_number: int, wire_type: int) -> int: ...
def UnpackTag(tag: int) -> tuple[int, int]: ...
def ZigZagEncode(value: int) -> int: ...
def ZigZagDecode(value: int) -> int: ...
def Int32ByteSize(field_number: int, int32: int) -> int: ...
def Int32ByteSizeNoTag(int32: int) -> int: ...
def Int64ByteSize(field_number: int, int64: int) -> int: ...
def UInt32ByteSize(field_number: int, uint32: int) -> int: ...
def UInt64ByteSize(field_number: int, uint64: int) -> int: ...
def SInt32ByteSize(field_number: int, int32: int) -> int: ...
def SInt64ByteSize(field_number: int, int64: int) -> int: ...
def Fixed32ByteSize(field_number: int, fixed32: int) -> int: ...
def Fixed64ByteSize(field_number: int, fixed64: int) -> int: ...
def SFixed32ByteSize(field_number: int, sfixed32: int) -> int: ...
def SFixed64ByteSize(field_number: int, sfixed64: int) -> int: ...
def FloatByteSize(field_number: int, flt: float) -> int: ...
def DoubleByteSize(field_number: int, double: float) -> int: ...
def BoolByteSize(field_number: int, b: bool) -> int: ...
def EnumByteSize(field_number: int, enum: int) -> int: ...
def StringByteSize(field_number: int, string: str) -> int: ...
def BytesByteSize(field_number: int, b: bytes) -> int: ...
def GroupByteSize(field_number: int, message: Message) -> int: ...
def MessageByteSize(field_number: int, message: Message) -> int: ...
def MessageSetItemByteSize(field_number: int, msg: Message) -> int: ...
def TagByteSize(field_number: int) -> int: ...

NON_PACKABLE_TYPES: tuple[int, ...]

def IsTypePackable(field_type: int) -> bool: ...



from _typeshed import Incomplete, StrPath, Unused
from collections.abc import Iterator, Mapping
from enum import Enum
from pathlib import Path
from types import TracebackType
from typing import ClassVar, Protocol, TypeAlias
from typing_extensions import Self

from .. import Command, errors, namespaces
from ..dist import Distribution
from ..warnings import SetuptoolsWarning

# Actually from wheel.wheelfile import WheelFile
_WheelFile: TypeAlias = Incomplete

class _EditableMode(Enum):
    STRICT = "strict"
    LENIENT = "lenient"
    COMPAT = "compat"
    @classmethod
    def convert(cls, mode: str | None) -> _EditableMode: ...

class editable_wheel(Command):
    description: str
    user_options: ClassVar[list[tuple[str, str | None, str]]]
    dist_dir: Incomplete
    dist_info_dir: Incomplete
    project_dir: Incomplete
    mode: Incomplete
    def initialize_options(self) -> None: ...
    package_dir: dict[Incomplete, Incomplete]
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...

class EditableStrategy(Protocol):
    def __call__(self, wheel: _WheelFile, files: list[str], mapping: Mapping[str, str]) -> Unused: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, _exc_type: type[BaseException] | None, _exc_value: BaseException | None, _traceback: TracebackType | None
    ) -> Unused: ...

class _StaticPth:
    dist: Distribution
    name: str
    path_entries: list[Path]
    def __init__(self, dist: Distribution, name: str, path_entries: list[Path]) -> None: ...
    def __call__(self, wheel: _WheelFile, files: list[str], mapping: Mapping[str, str]): ...
    def __enter__(self) -> Self: ...
    def __exit__(self, _exc_type: Unused, _exc_value: Unused, _traceback: Unused) -> None: ...

class _LinkTree(_StaticPth):
    auxiliary_dir: Path
    build_lib: Path
    def __init__(self, dist: Distribution, name: str, auxiliary_dir: StrPath, build_lib: StrPath) -> None: ...
    def __call__(self, wheel: _WheelFile, files: list[str], mapping: Mapping[str, str]): ...
    def __enter__(self) -> Self: ...
    def __exit__(self, _exc_type: Unused, _exc_value: Unused, _traceback: Unused) -> None: ...

class _TopLevelFinder:
    dist: Distribution
    name: str
    def __init__(self, dist: Distribution, name: str) -> None: ...
    def template_vars(self) -> tuple[str, str, dict[str, str], dict[str, list[str]]]: ...
    def get_implementation(self) -> Iterator[tuple[str, bytes]]: ...
    def __call__(self, wheel: _WheelFile, files: list[str], mapping: Mapping[str, str]): ...
    def __enter__(self) -> Self: ...
    def __exit__(self, _exc_type: Unused, _exc_value: Unused, _traceback: Unused) -> None: ...

class _NamespaceInstaller(namespaces.Installer):
    distribution: Incomplete
    src_root: Incomplete
    installation_dir: Incomplete
    editable_name: Incomplete
    outputs: list[str]
    def __init__(self, distribution, installation_dir, editable_name, src_root) -> None: ...

class InformationOnly(SetuptoolsWarning): ...
class LinksNotSupported(errors.FileError): ...



from _typeshed import StrOrBytesPath
from _typeshed.wsgi import StartResponse, WSGIEnvironment
from collections.abc import Iterable
from typing import TypedDict, type_check_only
from typing_extensions import Unpack

@type_check_only
class _DebugAppParams(TypedDict, total=False):
    form: StrOrBytesPath | bytes | None
    show_form: bool

__all__ = ["DebugApp", "make_debug_app"]

class DebugApp:
    form: bytes | None
    show_form: bool
    def __init__(self, form: StrOrBytesPath | bytes | None = None, show_form: bool = False) -> None: ...
    def __call__(self, environ: WSGIEnvironment, start_response: StartResponse) -> Iterable[bytes]: ...

debug_app: DebugApp

def make_debug_app(global_conf: object, **local_conf: Unpack[_DebugAppParams]) -> DebugApp: ...
